*
**********************
** BarCode Main Procedures
** BarCode Print routine
*     Parameters
*         BC_MESSG - information to be encoded and printed
*         BC_CPI - Character spacing value
*         BC_HEIGHT - BarCode Height - Inches for HPLaserJet
*                                      # of rows for DotMatrix
*         BC_PRTYPE - Printer Type
*         BC_PLPI - Printer Lines Per Inch

Procedure BC_Print
Parameters BC_Y,BC_X,BC_Msg,BC_Type,BC_Cpi,BC_Height,BC_PRType,BC_PLPI
Public BC_CStyle[36]
Private BC_Messg
*   16:  Static BC_MPrt,BC_SPrt,BC_GPrt,BC_GTPrt,BC_GMode

*    BC_CStyle Array for describing the BarCode Style
*    These may be preset with BC_Style procedure
*       BC_CStyle[1] BarCode type i.e. "CODE39"
*       BC_CStyle[2] X Coordinate
*       BC_CStyle[3] Y Coordinate
*       BC_CStyle[4] Message to encode
*       BC_CStyle[5] BarCode Verticle Alignment
*       BC_CStyle[6] BarCode Horizontal Alignment
*       BC_CStyle[7] Characters per inch
*       BC_CStyle[8] Height
*       BC_CStyle[9] Bearer Bars
*       BC_CStyle[10] Check Digit
*       BC_CStyle[11] Show Text
*       BC_CStyle[12] Text Position
*       BC_CStyle[13] Text Alignment
*       BC_CStyle[14] Text Size
*       BC_CStyle[15] Text Buffer Size
*       BC_CStyle[16] Printer Type
*       BC_CStyle[17] Printer LPI
*       BC_CStyle[18] Drop Distance
*       BC_CStyle[19] Drop Val
*       BC_CStyle[20] BAND Width
*       BC_CStyle[21] Pattern
*       BC_CStyle[22] TL VAL
*       BC_CStyle[23] TH VAL
*       BC_CStyle[24] Repeat
*       BC_CStyle[25] 1-Logical Port 2- Filename 3- HP/Postscript Macro
*       BC_CStyle[26] Port Destination
*       BC_CStyle[27] Output Filename
*       BC_CStyle[28] HP/Postscript Macro Number
*       BC_CStyle[29] Relative positioning
*       BC_CStyle[30] Cursors Final Location - ORIGIN or END
*       BC_CStyle[31] 1 - if setpage has been run
*       BC_CStyle[32] "L" - landscape "P" - Portrait
*       BC_CStyle[33] Relationship between space and bar default 1
*                     For UPC and Postnest barcodes this value must be 1
*       BC_CStyle[34] The final escape code to send to the printer after the
*                     BC is printed.
*       BC_CStyle[35] 1 - if FPREPORT is set, 0 - if not
*       BC_CStyle[36] UPC standard UCC code 0 - normal, 1 - UCC code

BC_CStyle[1] = iif(!empty(BC_Type),upper(BC_Type),;
               iif(!empty(BC_CStyle[1]), BC_CStyle[1],"CODE39"))
BC_CStyle[2] = iif(!empty(BC_X),BC_X,;
               iif(!empty(BC_CStyle[2]), BC_CStyle[2],0))
BC_CStyle[3] = iif(!empty(BC_Y),BC_Y,;
               iif(!empty(BC_CStyle[3]), BC_CStyle[3],0))
BC_CStyle[4] = iif(!empty(BC_Msg),upper(BC_Msg),;
               iif(!empty(BC_CStyle[4]), BC_CStyle[4],""))
BC_Messg = BC_CStyle[4]  && Assign to new var so changes don't effect original
BC_CStyle[7] = iif(!empty(BC_Cpi),BC_Cpi,;
               iif(!empty(BC_CStyle[7]),BC_CStyle[7],0))
BC_CStyle[8] = iif(!empty(BC_Height),BC_Height,;
               iif(!empty(BC_CStyle[8]), BC_CStyle[8],0))
BC_CStyle[16] = iif(!empty(BC_PRType),upper(BC_PRType),;
               iif(!empty(BC_CStyle[16]), BC_CStyle[16],"HPLASER"))
BC_CStyle[17] = iif(!empty(BC_PLPI),BC_PLPI,;
               iif(!empty(BC_CStyle[17]), BC_CStyle[17],6))
BC_CStyle[10] = iif(type("BC_CStyle[10]") = "N",BC_CStyle[10],1)
BC_CStyle[25] = iif(!empty(BC_CStyle[25]),BC_CStyle[25],1)
BC_CStyle[26] = iif(!empty(BC_CStyle[26]),BC_CStyle[26],"LPT1")
BC_CStyle[27] = iif(!empty(BC_CStyle[27]),BC_CStyle[27],"BC_OUT.PRN")
BC_CStyle[28] = iif(!empty(BC_CStyle[28]),BC_CStyle[28],1001)
BC_CStyle[29] = iif(type("BC_CStyle[29]") = "N",BC_CStyle[29],0)
BC_CStyle[30] = iif(!empty(BC_CStyle[30]),BC_CStyle[30],"END")
BC_CStyle[31] = iif(!empty(BC_CStyle[31]),BC_CStyle[31],0)
BC_CStyle[32] = iif(!empty(BC_CStyle[32]),BC_CStyle[32],"P")
BC_CStyle[33] = iif(!empty(BC_CStyle[33]),BC_CStyle[33],1)
BC_CStyle[34] = iif(type("BC_CStyle[34]")="C",;
                iif(len(BC_CStyle[34])>0,BC_CStyle[34],""),"")
BC_CStyle[35] = iif(!empty(BC_CStyle[35]),BC_CStyle[35],0)
BC_CStyle[36] = iif(!empty(BC_CStyle[36]),BC_CStyle[36],0)

Set Talk Off
* Set defaults if parameters are blank

* The laser support for barcode uses a subset of the LaserPro product.
* If using laser printer establish Page setup.
do case
  case BC_CStyle[25] >= 1 .and. BC_CStyle[25] < 3
    FP_ = BC_CStyle[25]
  otherwise
    FP_ = 1
endcase

if (BC_CStyle[16] = "HPLASER" .or. BC_CStyle[16] = "POSTSCRIPT") .and. BC_CStyle[31] <> 1
  null = LP_SetPage(iif(BC_CStyle[16] = "HPLASER",.t.,.f.),;
                    BC_CStyle[32], FP_, BC_CStyle[27], 1, 11, BC_CStyle[26])
  BC_CStyle[31] = 1
else
  Public LP_FP_
  LP_FP_ = FP_
endif

do case
  case BC_CStyle[1] = "CODE39"
    BC_CStyle[5] = iif(empty(BC_CStyle[5]),"TOP",BC_CStyle[5])
    BC_CStyle[6] = iif(empty(BC_CStyle[6]),"LEFT",BC_CStyle[6])
    BC_CStyle[7] = iif(!empty(BC_CStyle[7]) .and. BC_CStyle[7] <> 0,;
                       BC_CStyle[7],5.00)
    BC_CStyle[8] = iif(BC_CStyle[16] = "HPLASER";
                       .or. BC_CStyle[16] = "POSTSCRIPT",;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],.50),;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],4))
    BC_CStyle[12] = iif(empty(BC_CStyle[12]),"BELOW",BC_CStyle[12])
    BC_CStyle[13] = iif(empty(BC_CStyle[13]),"CENTER",BC_CStyle[13])
    BC_CStyle[15] = iif(empty(BC_CStyle[15]),iif(BC_CStyle[16] = "HPLASER" ;
                       .or. BC_CStyle[16] = "POSTSCRIPT",.025,.05),BC_CStyle[15])
    BC_CStyle[20] = 1/(14.5*BC_CStyle[7])
    BC_CStyle[21] = BC_LPCode39(upper(BC_Messg))
  case BC_CStyle[1] = "INTERLEAVED25"
    * Add check digit
    BC_Messg = iif(BC_CStyle[10] = 1,BC_InterlCheck(BC_Messg),BC_Messg)
    * Make sure there are an even number of characters if not add "0" to beginning
    BC_Messg = iif(mod(len(BC_Messg),2)>0,;
                       "0"+BC_Messg,BC_Messg)
    BC_CStyle[5] = iif(empty(BC_CStyle[5]),"TOP",BC_CStyle[5])
    BC_CStyle[6] = iif(empty(BC_CStyle[6]),"LEFT",BC_CStyle[6])
    BC_CStyle[7] = iif(!empty(BC_CStyle[7]) .and. BC_CStyle[7] <> 0,;
                       BC_CStyle[7],5.00)
    BC_CStyle[8] = iif(BC_CStyle[16] = "HPLASER";
                      .or. BC_CStyle[16] = "POSTSCRIPT",;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],.50),;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],4))
    BC_CStyle[12] = iif(empty(BC_CStyle[12]),"BELOW",BC_CStyle[12])
    BC_CStyle[13] = iif(empty(BC_CStyle[13]),"CENTER",BC_CStyle[13])
    BC_CStyle[15] = iif(empty(BC_CStyle[15]),iif(BC_CStyle[16] = "HPLASER";
                     .or. BC_CStyle[16] = "POSTSCRIPT",.025,.05),BC_CStyle[15])
    BC_CStyle[20] = 1/(16*(BC_CStyle[7]/2))
    BC_CStyle[21] = BC_Interl25(upper(BC_Messg))
  case BC_CStyle[1] = "CODE128"
    BC_CStyle[5] = iif(empty(BC_CStyle[5]),"TOP",BC_CStyle[5])
    BC_CStyle[6] = iif(empty(BC_CStyle[6]),"LEFT",BC_CStyle[6])
    BC_CStyle[7] = iif(!empty(BC_CStyle[7]) .and. BC_CStyle[7] <> 0,;
                       BC_CStyle[7],5.00)
    BC_CStyle[8] = iif(BC_CStyle[16] = "HPLASER" ;
                        .or. BC_CStyle[16] = "POSTSCRIPT",;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],.50),;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],4))
    BC_CStyle[12] = iif(empty(BC_CStyle[12]),"BELOW",BC_CStyle[12])
    BC_CStyle[13] = iif(empty(BC_CStyle[13]),"CENTER",BC_CStyle[13])
    BC_CStyle[15] = iif(empty(BC_CStyle[15]),iif(BC_CStyle[16] = "HPLASER" ;
                     .or. BC_CStyle[16] = "POSTSCRIPT",.025,.05),BC_CStyle[15])
    BC_CStyle[20] = 1/(11*BC_CStyle[7])
    BC_CStyle[21] = BC_CCVT128(BC_Messg)
  case BC_CStyle[1] = "UPC_A"
    * Add Check Digit
    BC_Messg = iif(BC_CStyle[10] = 1,BC_UPCheck(BC_Messg),BC_Messg)
    BC_CStyle[5] = iif(empty(BC_CStyle[5]),"TOP",BC_CStyle[5])
    BC_CStyle[6] = iif(empty(BC_CStyle[6]),"LEFT",BC_CStyle[6])
    BC_CStyle[7] = iif(!empty(BC_CStyle[7]) .and. BC_CStyle[7] <> 0,;
                       BC_CStyle[7],4.00)
    BC_CStyle[8] = iif(BC_CStyle[16] = "HPLASER";
                      .or. BC_CStyle[16] = "POSTSCRIPT",;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],.50),;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],4))
    BC_CStyle[12] = iif(empty(BC_CStyle[12]),"BELOW",BC_CStyle[12])
    BC_CStyle[13] = iif(empty(BC_CStyle[13]),"SPLIT",BC_CStyle[13])
    BC_CStyle[15] = iif(empty(BC_CStyle[15]),iif(BC_CStyle[16] = "HPLASER";
                     .or. BC_CStyle[16] = "POSTSCRIPT",.025,.05),BC_CStyle[15])
    BC_CStyle[20] = BC_CStyle[7]/95
    BC_CStyle[21] = BC_LPUPC(BC_Messg)
  case BC_CStyle[1] = "UPC_E"
    BC_CStyle[5] = iif(empty(BC_CStyle[5]),"TOP",BC_CStyle[5])
    BC_CStyle[6] = iif(empty(BC_CStyle[6]),"LEFT",BC_CStyle[6])
    BC_CStyle[7] = iif(!empty(BC_CStyle[7]) .and. BC_CStyle[7] <> 0,;
                       BC_CStyle[7],4.00)
    BC_CStyle[8] = iif(BC_CStyle[16] = "HPLASER";
                      .or. BC_CStyle[16] = "POSTSCRIPT",;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],.50),;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],4))
    BC_CStyle[12] = iif(empty(BC_CStyle[12]),"BELOW",BC_CStyle[12])
    BC_CStyle[13] = iif(empty(BC_CStyle[13]),"CENTER",BC_CStyle[13])
    BC_CStyle[15] = iif(empty(BC_CStyle[15]),iif(BC_CStyle[16] = "HPLASER";
                    .or. BC_CStyle[16] = "POSTSCRIPT",.025,.05),BC_CStyle[15])
    BC_CStyle[20] = BC_CStyle[7]/51
    BC_CStyle[21] = BC_LPUPC(BC_Messg)
  case BC_CStyle[1] = "POSTNET"
    * Add Check Digit
    BC_Messg = iif(BC_CStyle[10] = 1,BC_PNCheck(BC_Messg),BC_Messg)
    BC_CStyle[5] = iif(empty(BC_CStyle[5]),"BOTTOM",BC_CStyle[5])
    BC_CStyle[6] = iif(empty(BC_CStyle[6]),"LEFT",BC_CStyle[6])
    BC_CStyle[7] = iif(!empty(BC_CStyle[7]) .and. BC_CStyle[7] <> 0,;
                       BC_CStyle[7],0.05)
    BC_CStyle[8] = iif(BC_CStyle[16] = "HPLASER";
                      .or. BC_CStyle[16] = "POSTSCRIPT",;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],.135),;
                iif(!empty(BC_CStyle[8]) .and. BC_CStyle[8] <> 0,;
                    BC_CStyle[8],1))
    BC_CStyle[12] = iif(empty(BC_CStyle[12]),"BELOW",BC_CStyle[12])
    BC_CStyle[20] = BC_CStyle[7]/2
    BC_CStyle[21] = BC_LPPostNet(upper(BC_Messg))
endcase

BC_CStyle[9] = iif(type("BC_CStyle[9]") = "N",BC_CStyle[9],0)
BC_CStyle[11] = iif(type("BC_CStyle[11]") = "N",BC_CStyle[11],1)
BC_CStyle[14] = iif(!empty(BC_CStyle[14]),BC_CStyle[14],.12)
if BC_CStyle[16] = "HPLASER" .or. BC_CStyle[16] = "POSTSCRIPT"
  BC_CStyle[22] = iif(BC_CStyle[11] = 1 .and. BC_CStyle[1] <> "POSTNET",;
                  BC_CStyle[8] - BC_CStyle[14] - BC_CStyle[15],;
                  BC_CStyle[8])
  BC_CStyle[18] = iif(!empty(BC_CStyle[18]),BC_CStyle[18],180)
  BC_CStyle[19] = iif(!empty(BC_CStyle[19]),BC_CStyle[18],30)
  BC_CStyle[23] = iif(!empty(BC_CStyle[23]),BC_CStyle[23],23)
else
  BC_CStyle[18] = iif(BC_CStyle[16] = "EPSON",180,216)
  do case
    case BC_CStyle[17] = 6 .and. BC_CStyle[18] = 180
      BC_CStyle[19] = 30 && total down shifts per line
      BC_CStyle[23] = 23 && down shifts for main text part
      BC_CStyle[22] = 7  && down shifts for space text part
      BC_MPrt = 255      && print full band
      BC_SPrt = 7        && partial band for space text part
      BC_GPrt = 15       && partial bottom band for Guard bars
      BC_GTPrt = 240     && partial top band for Guard bars
    case BC_CStyle[17] = 8 .and. BC_CStyle[18] = 180
      BC_CStyle[19] = 22
      BC_CStyle[23] = 17
      BC_CStyle[22] = 5
      BC_MPrt = 255
      BC_SPrt = 7
      BC_GPrt = 15       && partial bottom band for Guard bars
      BC_GTPrt = 240     && partial top band for Guard bars
    case BC_CStyle[17] = 6 .and. BC_CStyle[18] = 216
      BC_CStyle[19] = 36
      BC_CStyle[23] = 23
      BC_CStyle[22] = 13
      BC_MPrt = 255
      BC_SPrt = 15
      BC_GPrt = 15       && partial bottom band for Guard bars
      BC_GTPrt = 240     && partial top band for Guard bars
    case BC_CStyle[17] = 8 .and. BC_CStyle[18] = 216
      BC_CStyle[19] = 27
      BC_CStyle[23] = 20
      BC_CStyle[22] = 7
      BC_MPrt = 255
      BC_SPrt = 15
      BC_GPrt = 15       && partial bottom band for Guard bars
      BC_GTPrt = 240     && partial top band for Guard bars
  endcase
endif

BC_CStyle[24] = iif(!empty(BC_CStyle[24]),BC_CStyle[24],2)

do case
  case BC_CStyle[16] = "HPLASER" .or. BC_CStyle[16] = "POSTSCRIPT"

    if BC_CStyle[25] = 3
      null = L_MacroID(BC_CStyle[28])
      null = L_MacroCtl(0)
    endif

    * Save current position to print the text
    if BC_CStyle[30] = "ORIGIN"
      null = LP_SRCurPos(.t.)
    endif

    null = BC_HPPBC()
    if BC_CStyle[11] = 1 .and. BC_CStyle[1] <> "POSTNET"
      null = BC_HPPTXT()
    endif

    * Restore current position to print the text
    if BC_CStyle[30] = "ORIGIN"
      null = LP_SRCurPos(.f.)
    endif

    if BC_CStyle[25] = 3
      null = L_MacroID(BC_CStyle[28])
      null = L_MacroCtl(1)
      null = L_MacroCtl(10)
    endif

  case BC_CStyle[16] = "EPSON" .or. BC_CStyle[16] = "IBMPRO"
    * if using FoxPro report writer increment the printer line by the height
    if BC_CStyle[35] = 1
      _PLINENO = _PLINENO + BC_CStyle[8] + iif(chr(10) $ BC_CStyle[34],1,0)
    endif
    null = BC_EPSPBC()
    if BC_CStyle[11] = 1 .and. BC_CStyle[1] <> "POSTNET"
      null = BC_EPSPTXT()
    endif
endcase
null = LP_Out(BC_CStyle[34])

return ""

*********************************************************************

Procedure BC_Style
Parameters BC_SType,BC_SVal
Public BC_CStyle[36]

do case
  case upper(BC_SType) = "TYPE"   && CODE39,UPC_A,UPC_E,CODE128,INTERLEAVED25,POSTNET
    BC_CStyle[1] = upper(BC_SVal)
  case upper(BC_SType) = "X_COORDINATE" .or. upper(BC_SType) = "COLUMN"
    BC_CStyle[2] = BC_SVal
  case upper(BC_SType) = "Y_COORDINATE" .or. upper(BC_SType) = "ROW"
      BC_CStyle[3] = BC_SVal
  case upper(BC_SType) = "MESSAGE"
    BC_CStyle[4] = upper(BC_SVal)
  case upper(BC_SType) = "VERTICAL_BAR_ALIGNMENT" && TOP,BOTTOM
    BC_CStyle[5] = upper(BC_SVal)
  case upper(BC_SType) = "HORIZONTAL_BAR_ALIGNMENT" && RIGHT,LEFT
    BC_CStyle[6] = upper(BC_SVal)
  case upper(BC_SType) = "CPI"
    BC_CStyle[7] = BC_SVal
  case upper(BC_SType) = "HEIGHT"
    BC_CStyle[8] = BC_SVal
  case upper(BC_SType) = "BEARER_BARS"  && && 1 - true, 0 - false
    BC_CStyle[9] = BC_SVal
  case upper(BC_SType) = "CHECK_DIGIT"  && 1 - true, 0 - false
    BC_CStyle[10] = BC_SVal
  case upper(BC_SType) = "SHOW_TEXT"  && && 1 - true, 0 - false
    BC_CStyle[11] = BC_SVal
  case upper(BC_SType) = "VERTICAL_TEXT_ALIGNMENT" && ABOVE, BELOW
    BC_CStyle[12] = upper(BC_SVal)
  case upper(BC_SType) = "HORIZONTAL_TEXT_ALIGNMENT" && CENTER,RIGHT,SPLIT
    BC_CStyle[13] = upper(BC_SVal)
  case upper(BC_SType) = "TEXT_SIZE"
    BC_CStyle[14] = BC_SVal
  case upper(BC_SType) = "TEXT_BUFFER" && distance between BC and Text
    BC_CStyle[15] = BC_SVal
  case upper(BC_SType) = "PRINTER_TYPE" && EPSON,IBMPRO,HPLASER,POSTSCRIPT
    BC_CStyle[16] = upper(BC_SVal)
  case upper(BC_SType) = "LPI"  && # of lines per inch on a dot matrix printer
    BC_CStyle[17] = BC_SVal
  case upper(BC_SType) = "DROPVAL"  && # of lines per inch on a dot matrix printer
    BC_CStyle[19] = BC_SVal
  case upper(BC_SType) = "REPEAT" && # of times to repeat on a dot matrix printer
    BC_CStyle[24] = BC_SVal
  case upper(BC_SType) = "OUTPUT" && location of output 1-Logical Port 2- File 3-Macro
    BC_CStyle[25] = BC_SVal
  case upper(BC_SType) = "PORT_DESTINATION" && LPT1, LPT2, etc
    BC_CStyle[26] = BC_SVal
  case upper(BC_SType) = "FILENAME" && Dos file name for Output to file
    BC_CStyle[27] = BC_SVal
  case upper(BC_SType) = "MACRONUMBER" && HP Macro number
    BC_CStyle[28] = BC_SVal
  case upper(BC_SType) = "RELATIVEPOS" && Relative positioning 1-true,0-false
    BC_CStyle[29] = BC_SVal
  case upper(BC_SType) = "CURSORPOS" && Cursor Final Position
    BC_CStyle[30] = BC_SVal
  case upper(BC_SType) = "ORIENTATION"
    BC_CStyle[32] = BC_SVal
  case upper(BC_SType) = "BAR/SPACE"
    BC_CStyle[33] = BC_SVal
  case upper(BC_SType) = "LASTCHR"
    BC_CStyle[34] = BC_SVal
  case upper(BC_SType) = "UCC_CODE"
    BC_CStyle[36] = BC_SVal
  case upper(BC_SType) = "CLEAR"  && Clear previous presets
    Release BC_CStyle
  case upper(BC_SType) = "FPREPORT"
     set talk off
     null=BC_Style("RELATIVEPOS",1)
     null=BC_Style("VERTICAL_BAR_ALIGNMENT","TOP")
*     null=BC_Style("LASTCHR",chr(13)+chr(10))
     null=BC_Style("LASTCHR","")
     BC_CStyle[35] = 1
endcase
return ""
*********************************************************************
*********************************************************************
* Code 39 5 Code Section
*********************************************************************

Procedure CCode39
Parameters BC_GMode,pval,codestr

codestr = ""
BC_GCOls = 0
for k = 1 TO len(BC_CStyle[21])
   do case
     case substr(BC_CStyle[21],k,1) = chr(0)
       null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(1)
       null=MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(2)
       null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(3)
       null=MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
  endcase
endfor

return BC_GCols

*********************************************************************

Procedure BC_LPCode39
Parameters m
private ns,ws,nb,wb,wb1,wb2,wb3,wb4
private ws1,ws2,ws3,nb1,nb2,nb3,nb4
private ns1,ns2,ns3,ns4,msg,t,x,p

* Generates bar pattern for code 39 bar codes
*    m  = text to decode into a bit pattern
*
* Returns:
*    Bar pattern for code 39 bar code.
*    If an illegal value is found then a null will
*    be returned to the user.

*** if the parameter is not a character then quit
if type('m')<>"C"
  return ""
endif

*** Check to see if user added stop/start
*** character and returns null if true
if at("*"," "+m)>1
  return ""
endif

*** add start and stop characters to message
msg = "*" + upper(m) + "*"

*** define narrow/wide spaces/bars
ns = chr(0)
ws = chr(1)
nb = chr(2)
wb = chr(3)

*** Create the bit patterns we may use
WB1 = wb + ns + nb
WB2 = wb + ns + wb
WB3 = wb + ws + wb
WB4 = wb + ws + nb

WS1 = ws + nb + ns
WS2 = ws + wb + ns
WS3 = ws + nb + ws

NB1 = nb + ns + wb
NB2 = nb + ns + nb
NB3 = nb + ws + wb
NB4 = nb + ws + nb

NS1 = ns + wb + ws
NS2 = ns + nb + ws
NS3 = ns + nb + ns
NS4 = ns + wb + ns

*** decode message into bar code bit pattern and place
*** it into the variable
p=""

FOR x = 1 TO len(msg)
  t = substr(msg, x, 1)

  do case
    CASE t = "1"
      p = p + WB1 + WS1 + NB1
    CASE t = "2"
      p = p + NB1 + WS1 + NB1
    CASE t = "3"
      p = p + WB2 + WS1 + NB2
    CASE t = "4"
      p = p + NB2 + WS2 + NB1
    CASE t = "5"
      p = p + WB1 + WS2 + NB2
    CASE t = "6"
      p = p + NB1 + WS2 + NB2
    CASE t = "7"
      p = p + NB2 + WS1 + WB2
    CASE t = "8"
      p = p + WB1 + WS1 + WB1
    CASE t = "9"
      p = p + NB1 + WS1 + WB1
    CASE t = "0"
      p = p + NB2 + WS2 + WB1
    CASE t = "A"
      p = p + WB1 + NS2 + NB1
    CASE t = "B"
      p = p + NB1 + NS2 + NB1
    CASE t = "C"
      p = p + WB2 + NS2 + NB2
    CASE t = "D"
      p = p + NB2 + NS1 + NB1
    CASE t = "E"
      p = p + WB1 + NS1 + NB2
    CASE t = "F"
      p = p + NB1 + NS1 + NB2
    CASE t = "G"
      p = p + NB2 + NS2 + WB2
    CASE t = "H"
      p = p + WB1 + NS2 + WB1
    CASE t = "I"
      p = p + NB1 + NS2 + WB1
    CASE t = "J"
      p = p + NB2 + NS1 + WB1
    CASE t = "K"
      p = p + WB1 + NS3 + NB3
    CASE t = "L"
      p = p + NB1 + NS3 + NB3
    CASE t = "M"
      p = p + WB2 + NS3 + NB4
    CASE t = "N"
      p = p + NB2 + NS4 + NB3
    CASE t = "O"
      p = p + WB1 + NS4 + NB4
    CASE t = "P"
      p = p + NB1 + NS4 + NB4
    CASE t = "Q"
      p = p + NB2 + NS3 + WB3
    CASE t = "R"
      p = p + WB1 + NS3 + WB4
    CASE t = "S"
      p = p + NB1 + NS3 + WB4
    CASE t = "T"
      p = p + NB2 + NS4 + WB4
    CASE t = "U"
      p = p + WB4 + NS3 + NB1
    CASE t = "V"
      p = p + NB3 + NS3 + NB1
    CASE t = "W"
      p = p + WB3 + NS3 + NB2
    CASE t = "X"
      p = p + NB4 + NS4 + NB1
    CASE t = "Y"
      p = p + WB4 + NS4 + NB2
    CASE t = "Z"
      p = p + NB3 + NS4 + NB2
    CASE t = "-"
      p = p + NB4 + NS3 + WB2
    CASE t = "."
      p = p + WB4 + NS3 + WB1
    CASE t = " "
      p = p + NB3 + NS3 + WB1
    CASE t = "*"
      p = p + NB4 + NS4 + WB1
    CASE t = "$"
      p = p + NB4 + WS3 + NB2
    CASE t = "/"
      p = p + NB4 + WS1 + NB4
    CASE t = "+"
      p = p + NB4 + NS2 + NB4
    CASE t = "%"
      p = p + NB2 + WS3 + NB4
    otherwise
      *** Invalid code found
      return ""
  endcase

  *** Each bar code has a narrow space after it
  p=p+ns
endfor

*** Return the bit pattern
return p

*********************************************************************

*********************************************************************
* Interleave 2 of 5 Code Section
*********************************************************************

Procedure BC_InterlCheck
Parameter msg

* Calculate and add the check character *
bcc = 0
bco = 0
for i = 1 to len(msg)
  bco = bco + val(substr(msg, i, 1))
endfor

bcc = mod(bco,10)

*** Finish the UPC symbol.  Add the checksum character
msg = msg + str(bcc, 1)
return msg

*********************************************************************

Procedure INTERL25
Parameters BC_GMode,pval,codestr

codestr = ""
BC_GCols = 0
for k = 1 TO len(BC_CStyle[21])
   do case
     case substr(BC_CStyle[21],k,1) = chr(0)
       null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(1)
       null=MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(2)
       null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(3)
       null=MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
  endcase
endfor

return BC_GCols

*********************************************************************

Procedure BC_Interl25
Parameters m
private ns,ws,nb,wb
private t,x,t1,odd_m,even_m

* Generates bar pattern for Interleaved 2 of 5 bar codes
*    m  = text to decode into a bit pattern
*
* Returns:
*    Bar pattern for Interleaved 2 of 5 bar code.
*    If an illegal value is found then a null will
*    be returned to the user.

*** if the parameter is not a character then quit
if type('m')<>"C"
  return ""
endif

*** define narrow/wide spaces/bars
ns = chr(0)
ws = chr(1)
nb = chr(2)
wb = chr(3)

odd_m = ""
even_m = ""

*** decode message into bar code bit pattern and place
*** it into the variable
* Start with start code
p = nb + ns + nb + ns

FOR x = 1 TO len(m) step 2
  t = substr(m, x, 1)
  t1 = substr(m, x+1, 1)

  for y = 1 to 5
    do case
      CASE t = "0"
        odd_m = substr(nb + nb + wb + wb + nb , y, 1)
      CASE t = "1"
        odd_m = substr(wb + nb + nb + nb + wb , y, 1)
      CASE t = "2"
        odd_m = substr(nb + wb + nb + nb + wb , y, 1)
      CASE t = "3"
        odd_m = substr(wb + wb + nb + nb + nb , y, 1)
      CASE t = "4"
        odd_m = substr(nb + nb + wb + nb + wb , y, 1)
      CASE t = "5"
        odd_m = substr(wb + nb + wb + nb + nb , y, 1)
      CASE t = "6"
        odd_m = substr(nb + wb + wb + nb + nb , y, 1)
      CASE t = "7"
        odd_m = substr(nb + nb + nb + wb + wb , y, 1)
      CASE t = "8"
        odd_m = substr(wb + nb + nb + wb + nb , y, 1)
      CASE t = "9"
        odd_m = substr(nb + wb + nb + wb + nb , y, 1)
      OTHERWISE
        *** Invalid code found
        return ""
    endcase
    do case
      CASE t1 = "0"
        even_m = substr(ns + ns + ws + ws + ns , y, 1)
      CASE t1 = "1"
        even_m = substr(ws + ns + ns + ns + ws , y, 1)
      CASE t1 = "2"
        even_m = substr(ns + ws + ns + ns + ws , y, 1)
      CASE t1 = "3"
        even_m = substr(ws + ws + ns + ns + ns , y, 1)
      CASE t1 = "4"
        even_m = substr(ns + ns + ws + ns + ws , y, 1)
      CASE t1 = "5"
        even_m = substr(ws + ns + ws + ns + ns , y, 1)
      CASE t1 = "6"
        even_m = substr(ns + ws + ws + ns + ns , y, 1)
      CASE t1 = "7"
        even_m = substr(ns + ns + ns + ws + ws , y, 1)
      CASE t1 = "8"
        even_m = substr(ws + ns + ns + ws + ns , y, 1)
      CASE t1 = "9"
        even_m = substr(ns + ws + ns + ws + ns , y, 1)
      OTHERWISE
        *** Invalid code found
        return ""
    endcase

    p = p + odd_m + even_m
  endfor
endfor

*** Add stop code
p = p + wb + ns + nb

*** Return the bit pattern
return p

*********************************************************************

*********************************************************************
* Code 128 Code Section
*********************************************************************

Procedure CCode128
parameters BC_GMode,pval,codestr

codestr = ""
BC_GCOls = 0
for k = 1 TO len(BC_CStyle[21])
   do case
     case substr(BC_CStyle[21],k,1) = chr(0)
       null = MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(1)
       null = MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(2)
       null = MkCodeStr(@codestr,.t.,3,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(3)
       null = MkCodeStr(@codestr,.t.,4,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(4)
       null = MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(5)
       null = MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(6)
       null = MkCodeStr(@codestr,.f.,3,@BC_GCols,pval)
     case substr(BC_CStyle[21],k,1) = chr(7)
       null = MkCodeStr(@codestr,.f.,4,@BC_GCols,pval)
  endcase
endfor

return BC_GCols

*********************************************************************

Procedure BC_128CHR
Parameters ch
s_char = chr(128)
return s_char+s_char+chr(ch-96)

*********************************************************************

Procedure BC_CCVT128
Parameters m

codeset = 1
cset =1
shift = .f.
fcn4 = 1
pfcn4 = .f.
cont = .t.
dimension cvtarr[4,100]
dimension contarr[2,10]
s_char = chr(128)
cnt = 1
rtncode = ""

* Define the convertion array
cvtarr[1,1]="000"
cvtarr[2,1]=chr(32)
cvtarr[3,1]=chr(32)
cvtarr[4,1]="00"
cvtarr[1,2]="001"
cvtarr[2,2]=chr(33)
cvtarr[3,2]=chr(33)
cvtarr[4,2]="01"
cvtarr[1,3]="002"
cvtarr[2,3]=chr(34)
cvtarr[3,3]=chr(34)
cvtarr[4,3]="02"
cvtarr[1,4]="003"
cvtarr[2,4]=chr(35)
cvtarr[3,4]=chr(35)
cvtarr[4,4]="03"
cvtarr[1,5]="004"
cvtarr[2,5]=chr(36)
cvtarr[3,5]=chr(36)
cvtarr[4,5]="04"
cvtarr[1,6]="005"
cvtarr[2,6]=chr(37)
cvtarr[3,6]=chr(37)
cvtarr[4,6]="05"
cvtarr[1,7]="006"
cvtarr[2,7]=chr(38)
cvtarr[3,7]=chr(38)
cvtarr[4,7]="06"
cvtarr[1,8]="007"
cvtarr[2,8]=chr(39)
cvtarr[3,8]=chr(39)
cvtarr[4,8]="07"
cvtarr[1,9]="008"
cvtarr[2,9]=chr(40)
cvtarr[3,9]=chr(40)
cvtarr[4,9]="08"
cvtarr[1,10]="009"
cvtarr[2,10]=chr(41)
cvtarr[3,10]=chr(41)
cvtarr[4,10]="09"
cvtarr[1,11]="010"
cvtarr[2,11]=chr(42)
cvtarr[3,11]=chr(42)
cvtarr[4,11]="10"
cvtarr[1,12]="011"
cvtarr[2,12]=chr(43)
cvtarr[3,12]=chr(43)
cvtarr[4,12]="11"
cvtarr[1,13]="012"
cvtarr[2,13]=chr(44)
cvtarr[3,13]=chr(44)
cvtarr[4,13]="12"
cvtarr[1,14]="013"
cvtarr[2,14]=chr(45)
cvtarr[3,14]=chr(45)
cvtarr[4,14]="13"
cvtarr[1,15]="014"
cvtarr[2,15]=chr(46)
cvtarr[3,15]=chr(46)
cvtarr[4,15]="14"
cvtarr[1,16]="015"
cvtarr[2,16]=chr(47)
cvtarr[3,16]=chr(47)
cvtarr[4,16]="15"
cvtarr[1,17]="016"
cvtarr[2,17]=chr(48)
cvtarr[3,17]=chr(48)
cvtarr[4,17]="16"
cvtarr[1,18]="017"
cvtarr[2,18]=chr(49)
cvtarr[3,18]=chr(49)
cvtarr[4,18]="17"
cvtarr[1,19]="018"
cvtarr[2,19]=chr(50)
cvtarr[3,19]=chr(50)
cvtarr[4,19]="18"
cvtarr[1,20]="019"
cvtarr[2,20]=chr(51)
cvtarr[3,20]=chr(51)
cvtarr[4,20]="19"
cvtarr[1,21]="020"
cvtarr[2,21]=chr(52)
cvtarr[3,21]=chr(52)
cvtarr[4,21]="20"
cvtarr[1,22]="021"
cvtarr[2,22]=chr(53)
cvtarr[3,22]=chr(53)
cvtarr[4,22]="21"
cvtarr[1,23]="022"
cvtarr[2,23]=chr(54)
cvtarr[3,23]=chr(54)
cvtarr[4,23]="22"
cvtarr[1,24]="023"
cvtarr[2,24]=chr(55)
cvtarr[3,24]=chr(55)
cvtarr[4,24]="23"
cvtarr[1,25]="024"
cvtarr[2,25]=chr(56)
cvtarr[3,25]=chr(56)
cvtarr[4,25]="24"
cvtarr[1,26]="025"
cvtarr[2,26]=chr(57)
cvtarr[3,26]=chr(57)
cvtarr[4,26]="25"
cvtarr[1,27]="026"
cvtarr[2,27]=chr(58)
cvtarr[3,27]=chr(58)
cvtarr[4,27]="26"
cvtarr[1,28]="027"
cvtarr[2,28]=chr(59)
cvtarr[3,28]=chr(59)
cvtarr[4,28]="27"
cvtarr[1,29]="028"
cvtarr[2,29]=chr(60)
cvtarr[3,29]=chr(60)
cvtarr[4,29]="28"
cvtarr[1,30]="029"
cvtarr[2,30]=chr(61)
cvtarr[3,30]=chr(61)
cvtarr[4,30]="29"
cvtarr[1,31]="030"
cvtarr[2,31]=chr(62)
cvtarr[3,31]=chr(62)
cvtarr[4,31]="30"
cvtarr[1,32]="031"
cvtarr[2,32]=chr(63)
cvtarr[3,32]=chr(63)
cvtarr[4,32]="31"
cvtarr[1,33]="032"
cvtarr[2,33]=chr(64)
cvtarr[3,33]=chr(64)
cvtarr[4,33]="32"
cvtarr[1,34]="033"
cvtarr[2,34]=chr(65)
cvtarr[3,34]=chr(65)
cvtarr[4,34]="33"
cvtarr[1,35]="034"
cvtarr[2,35]=chr(66)
cvtarr[3,35]=chr(66)
cvtarr[4,35]="34"
cvtarr[1,36]="035"
cvtarr[2,36]=chr(67)
cvtarr[3,36]=chr(67)
cvtarr[4,36]="35"
cvtarr[1,37]="036"
cvtarr[2,37]=chr(68)
cvtarr[3,37]=chr(68)
cvtarr[4,37]="36"
cvtarr[1,38]="037"
cvtarr[2,38]=chr(69)
cvtarr[3,38]=chr(69)
cvtarr[4,38]="37"
cvtarr[1,39]="038"
cvtarr[2,39]=chr(70)
cvtarr[3,39]=chr(70)
cvtarr[4,39]="38"
cvtarr[1,40]="039"
cvtarr[2,40]=chr(71)
cvtarr[3,40]=chr(71)
cvtarr[4,40]="39"
cvtarr[1,41]="040"
cvtarr[2,41]=chr(72)
cvtarr[3,41]=chr(72)
cvtarr[4,41]="40"
cvtarr[1,42]="041"
cvtarr[2,42]=chr(73)
cvtarr[3,42]=chr(73)
cvtarr[4,42]="41"
cvtarr[1,43]="042"
cvtarr[2,43]=chr(74)
cvtarr[3,43]=chr(74)
cvtarr[4,43]="42"
cvtarr[1,44]="043"
cvtarr[2,44]=chr(75)
cvtarr[3,44]=chr(75)
cvtarr[4,44]="43"
cvtarr[1,45]="044"
cvtarr[2,45]=chr(76)
cvtarr[3,45]=chr(76)
cvtarr[4,45]="44"
cvtarr[1,46]="045"
cvtarr[2,46]=chr(77)
cvtarr[3,46]=chr(77)
cvtarr[4,46]="45"
cvtarr[1,47]="046"
cvtarr[2,47]=chr(78)
cvtarr[3,47]=chr(78)
cvtarr[4,47]="46"
cvtarr[1,48]="047"
cvtarr[2,48]=chr(79)
cvtarr[3,48]=chr(79)
cvtarr[4,48]="47"
cvtarr[1,49]="048"
cvtarr[2,49]=chr(80)
cvtarr[3,49]=chr(80)
cvtarr[4,49]="48"
cvtarr[1,50]="049"
cvtarr[2,50]=chr(81)
cvtarr[3,50]=chr(81)
cvtarr[4,50]="49"
cvtarr[1,51]="050"
cvtarr[2,51]=chr(82)
cvtarr[3,51]=chr(82)
cvtarr[4,51]="50"
cvtarr[1,52]="051"
cvtarr[2,52]=chr(83)
cvtarr[3,52]=chr(83)
cvtarr[4,52]="51"
cvtarr[1,53]="052"
cvtarr[2,53]=chr(84)
cvtarr[3,53]=chr(84)
cvtarr[4,53]="52"
cvtarr[1,54]="053"
cvtarr[2,54]=chr(85)
cvtarr[3,54]=chr(85)
cvtarr[4,54]="53"
cvtarr[1,55]="054"
cvtarr[2,55]=chr(86)
cvtarr[3,55]=chr(86)
cvtarr[4,55]="54"
cvtarr[1,56]="055"
cvtarr[2,56]=chr(87)
cvtarr[3,56]=chr(87)
cvtarr[4,56]="55"
cvtarr[1,57]="056"
cvtarr[2,57]=chr(88)
cvtarr[3,57]=chr(88)
cvtarr[4,57]="56"
cvtarr[1,58]="057"
cvtarr[2,58]=chr(89)
cvtarr[3,58]=chr(89)
cvtarr[4,58]="57"
cvtarr[1,59]="058"
cvtarr[2,59]=chr(90)
cvtarr[3,59]=chr(90)
cvtarr[4,59]="58"
cvtarr[1,60]="059"
cvtarr[2,60]=chr(91)
cvtarr[3,60]=chr(91)
cvtarr[4,60]="59"
cvtarr[1,61]="060"
cvtarr[2,61]=chr(92)
cvtarr[3,61]=chr(92)
cvtarr[4,61]="60"
cvtarr[1,62]="061"
cvtarr[2,62]=chr(93)
cvtarr[3,62]=chr(93)
cvtarr[4,62]="61"
cvtarr[1,63]="062"
cvtarr[2,63]=chr(94)
cvtarr[3,63]=chr(94)
cvtarr[4,63]="62"
cvtarr[1,64]="063"
cvtarr[2,64]=chr(95)
cvtarr[3,64]=chr(95)
cvtarr[4,64]="63"
cvtarr[1,65]="064"
cvtarr[2,65]=chr(0)
cvtarr[3,65]=chr(96)
cvtarr[4,65]="64"
cvtarr[1,66]="065"
cvtarr[2,66]=chr(1)
cvtarr[3,66]=chr(97)
cvtarr[4,66]="65"
cvtarr[1,67]="066"
cvtarr[2,67]=chr(2)
cvtarr[3,67]=chr(98)
cvtarr[4,67]="66"
cvtarr[1,68]="067"
cvtarr[2,68]=chr(3)
cvtarr[3,68]=chr(99)
cvtarr[4,68]="67"
cvtarr[1,69]="068"
cvtarr[2,69]=chr(4)
cvtarr[3,69]=chr(100)
cvtarr[4,69]="68"
cvtarr[1,70]="069"
cvtarr[2,70]=chr(5)
cvtarr[3,70]=chr(101)
cvtarr[4,70]="69"
cvtarr[1,71]="070"
cvtarr[2,71]=chr(6)
cvtarr[3,71]=chr(102)
cvtarr[4,71]="70"
cvtarr[1,72]="071"
cvtarr[2,72]=chr(7)
cvtarr[3,72]=chr(103)
cvtarr[4,72]="71"
cvtarr[1,73]="072"
cvtarr[2,73]=chr(8)
cvtarr[3,73]=chr(104)
cvtarr[4,73]="72"
cvtarr[1,74]="073"
cvtarr[2,74]=chr(9)
cvtarr[3,74]=chr(105)
cvtarr[4,74]="73"
cvtarr[1,75]="074"
cvtarr[2,75]=chr(10)
cvtarr[3,75]=chr(106)
cvtarr[4,75]="74"
cvtarr[1,76]="075"
cvtarr[1,76]=chr(11)
cvtarr[3,76]=chr(107)
cvtarr[4,76]="75"
cvtarr[1,77]="076"
cvtarr[2,77]=chr(12)
cvtarr[3,77]=chr(108)
cvtarr[4,77]="76"
cvtarr[1,78]="077"
cvtarr[2,78]=chr(13)
cvtarr[3,78]=chr(109)
cvtarr[4,78]="77"
cvtarr[1,79]="078"
cvtarr[2,79]=chr(14)
cvtarr[3,79]=chr(110)
cvtarr[4,79]="78"
cvtarr[1,80]="079"
cvtarr[2,80]=chr(15)
cvtarr[3,80]=chr(111)
cvtarr[4,80]="79"
cvtarr[1,81]="080"
cvtarr[2,81]=chr(16)
cvtarr[3,81]=chr(112)
cvtarr[4,81]="80"
cvtarr[1,82]="081"
cvtarr[2,82]=chr(17)
cvtarr[3,82]=chr(113)
cvtarr[4,82]="81"
cvtarr[1,83]="082"
cvtarr[2,83]=chr(18)
cvtarr[3,83]=chr(114)
cvtarr[4,83]="82"
cvtarr[1,84]="083"
cvtarr[2,84]=chr(19)
cvtarr[3,84]=chr(115)
cvtarr[4,84]="83"
cvtarr[1,85]="084"
cvtarr[2,85]=chr(20)
cvtarr[3,85]=chr(116)
cvtarr[4,85]="84"
cvtarr[1,86]="085"
cvtarr[2,86]=chr(21)
cvtarr[3,86]=chr(117)
cvtarr[4,86]="85"
cvtarr[1,87]="086"
cvtarr[2,87]=chr(22)
cvtarr[3,87]=chr(118)
cvtarr[4,87]="86"
cvtarr[1,88]="087"
cvtarr[2,88]=chr(23)
cvtarr[3,88]=chr(119)
cvtarr[4,88]="87"
cvtarr[1,89]="088"
cvtarr[2,89]=chr(24)
cvtarr[3,89]=chr(120)
cvtarr[4,89]="88"
cvtarr[1,90]="089"
cvtarr[2,90]=chr(25)
cvtarr[3,90]=chr(121)
cvtarr[4,90]="89"
cvtarr[1,91]="090"
cvtarr[2,91]=chr(26)
cvtarr[3,91]=chr(122)
cvtarr[4,91]="90"
cvtarr[1,92]="091"
cvtarr[2,92]=chr(27)
cvtarr[3,92]=chr(123)
cvtarr[4,92]="91"
cvtarr[1,93]="092"
cvtarr[2,93]=chr(28)
cvtarr[3,93]=chr(124)
cvtarr[4,93]="92"
cvtarr[1,94]="093"
cvtarr[2,94]=chr(29)
cvtarr[3,94]=chr(125)
cvtarr[4,94]="93"
cvtarr[1,95]="094"
cvtarr[2,95]=chr(30)
cvtarr[3,95]=chr(126)
cvtarr[4,95]="94"
cvtarr[1,96]="095"
cvtarr[2,96]=chr(31)
cvtarr[3,96]=chr(127)
cvtarr[4,96]="95"
cvtarr[1,97]="096"
cvtarr[2,97]=""
cvtarr[3,97]=""
cvtarr[4,97]="96"
cvtarr[1,98]="097"
cvtarr[2,98]=""
cvtarr[3,98]=""
cvtarr[4,98]="97"
cvtarr[1,99]="098"
cvtarr[2,99]=""
cvtarr[3,99]=""
cvtarr[4,99]="98"
cvtarr[1,100]="099"
cvtarr[2,100]=""
cvtarr[3,100]=""
cvtarr[4,100]="99"

contarr[1,1]="096"
contarr[2,1]=chr(0)
contarr[1,2]="097"
contarr[2,2]=chr(1)
contarr[1,3]="098"
contarr[2,3]=chr(2)
contarr[1,4]="099"
contarr[2,4]=chr(3)
contarr[1,5]="100"
contarr[2,5]=chr(4)
contarr[1,6]="101"
contarr[2,6]=chr(5)
contarr[1,7]="102"
contarr[2,7]=chr(6)
contarr[1,8]="103"
contarr[2,8]=chr(7)
contarr[1,9]="104"
contarr[2,9]=chr(8)
contarr[1,10]="105"
contarr[2,10]=chr(9)

do while cont
  t = substr(m,cnt,1)
  do case
    case t = s_char .and. substr(m,cnt+1,1) = s_char
      t = substr(m,cnt+2,1)
      rtnc = asubscript(contarr,ascan(contarr,t,aelement(contarr,2,1),10),2)
      if rtnc <> 0
        rtncode = rtncode + contarr[1,rtnc]
        shift = iif(contarr[2,rtnc] == chr(2),.t.,.f.)
        codeset = iif(contarr[2,rtnc] == chr(3),3,codeset)
        codeset = iif(contarr[2,rtnc] == chr(4) .and. codeset <> 2,2,codeset)
        codeset = iif(contarr[2,rtnc] == chr(5) .and. codeset <> 1,1,codeset)
        codeset = iif(contarr[2,rtnc] == chr(7),1,codeset)
        codeset = iif(contarr[2,rtnc] == chr(8),2,codeset)
        codeset = iif(contarr[2,rtnc] == chr(9),3,codeset)

        if (contarr[2,rtnc] = chr(4) .and. codeset = 2) .or. ;
           (contarr[2,rtnc] = chr(5) .and. codeset = 1)
           fcn4 = iif(fcn4 < 4,fcn4 + 1,1)
        endif
      endif
      cnt = cnt + 3

    case codeset = 3 .and. asc(t) >= asc("0") .and. asc(t) <= asc("9")
      t = substr(m,cnt,2)
      rtnc = asubscript(cvtarr,ascan(cvtarr,t,aelement(cvtarr,4,1),100),2)
      if rtnc <> 0
        rtncode = rtncode + cvtarr[1,rtnc]
      endif
      cnt = cnt + 2
      shift = .f.

    otherwise
      if fcn4 > 1
        if fcn4 = 4
          t = chr(asc(t) - 128)
        endif
        fcn4 = iif(fcn4 == 2,1,3)
      endif
      cset = iif(shift,iif(codeset == 1,2,1),codeset)
      rtnc = asubscript(cvtarr,ascan(cvtarr,t,aelement(cvtarr,cset+1,1),100),2)
      if rtnc <> 0
        rtncode = rtncode + cvtarr[1,rtnc]
      endif
      cnt = cnt + 1
      shift = .f.
  endcase
  cont = iif(cnt > len(m),.f.,.t.)
enddo

* convert the value codes to bars and spaces and add the check digit
rcode = bc_c128check(rtncode)

** return the bit pattern
return rcode

*********************************************************************

Procedure BC_c128check
Parameters m

dimension cnvarr[2,106]
j = 0
rtnstr = ""
chktot = 0

b1 = chr(0)
b2 = chr(1)
b3 = chr(2)
b4 = chr(3)
s1 = chr(4)
s2 = chr(5)
s3 = chr(6)
s4 = chr(7)

cnvarr[1,1]="000"
cnvarr[2,1]=b2+s1+b2+s2+b2+s2
cnvarr[1,2]="001"
cnvarr[2,2]=b2+s2+b2+s1+b2+s2
cnvarr[1,3]="002"
cnvarr[2,3]=b2+s2+b2+s2+b2+s1
cnvarr[1,4]="003"
cnvarr[2,4]=b1+s2+b1+s2+b2+s3
cnvarr[1,5]="004"
cnvarr[2,5]=b1+s2+b1+s3+b2+s2
cnvarr[1,6]="005"
cnvarr[2,6]=b1+s3+b1+s2+b2+s2
cnvarr[1,7]="006"
cnvarr[2,7]=b1+s2+b2+s2+b1+s3
cnvarr[1,8]="007"
cnvarr[2,8]=b1+s2+b2+s3+b1+s2
cnvarr[1,9]="008"
cnvarr[2,9]=b1+s3+b2+s2+b1+s2
cnvarr[1,10]="009"
cnvarr[2,10]=b2+s2+b1+s2+b1+s3
cnvarr[1,11]="010"
cnvarr[2,11]=b2+s2+b1+s3+b1+s2
cnvarr[1,12]="011"
cnvarr[2,12]=b2+s3+b1+s2+b1+s2
cnvarr[1,13]="012"
cnvarr[2,13]=b1+s1+b2+s2+b3+s2
cnvarr[1,14]="013"
cnvarr[2,14]=b1+s2+b2+s1+b3+s2
cnvarr[1,15]="014"
cnvarr[2,15]=b1+s2+b2+s2+b3+s1
cnvarr[1,16]="015"
cnvarr[2,16]=b1+s1+b3+s2+b2+s2
cnvarr[1,17]="016"
cnvarr[2,17]=b1+s2+b3+s1+b2+s2
cnvarr[1,18]="017"
cnvarr[2,18]=b1+s2+b3+s2+b2+s1
cnvarr[1,19]="018"
cnvarr[2,19]=b2+s2+b3+s2+b1+s1
cnvarr[1,20]="019"
cnvarr[2,20]=b2+s2+b1+s1+b3+s2
cnvarr[1,21]="020"
cnvarr[2,21]=b2+s2+b1+s2+b3+s1
cnvarr[1,22]="021"
cnvarr[2,22]=b2+s1+b3+s2+b1+s2
cnvarr[1,23]="022"
cnvarr[2,23]=b2+s2+b3+s1+b1+s2
cnvarr[1,24]="023"
cnvarr[2,24]=b3+s1+b2+s1+b3+s1
cnvarr[1,25]="024"
cnvarr[2,25]=b3+s1+b1+s2+b2+s2
cnvarr[1,26]="025"
cnvarr[2,26]=b3+s2+b1+s1+b2+s2
cnvarr[1,27]="026"
cnvarr[2,27]=b3+s2+b1+s2+b2+s1
cnvarr[1,28]="027"
cnvarr[2,28]=b3+s1+b2+s2+b1+s2
cnvarr[1,29]="028"
cnvarr[2,29]=b3+s2+b2+s1+b1+s2
cnvarr[1,30]="029"
cnvarr[2,30]=b3+s2+b2+s2+b1+s1
cnvarr[1,31]="030"
cnvarr[2,31]=b2+s1+b2+s1+b2+s3
cnvarr[1,32]="031"
cnvarr[2,32]=b2+s1+b2+s3+b2+s1
cnvarr[1,33]="032"
cnvarr[2,33]=b2+s3+b2+s1+b2+s1
cnvarr[1,34]="033"
cnvarr[2,34]=b1+s1+b1+s3+b2+s3
cnvarr[1,35]="034"
cnvarr[2,35]=b1+s3+b1+s1+b2+s3
cnvarr[1,36]="035"
cnvarr[2,36]=b1+s3+b1+s3+b2+s1
cnvarr[1,37]="036"
cnvarr[2,37]=b1+s1+b2+s3+b1+s3
cnvarr[1,38]="037"
cnvarr[2,38]=b1+s3+b2+s1+b1+s3
cnvarr[1,39]="038"
cnvarr[2,39]=b1+s3+b2+s3+b1+s1
cnvarr[1,40]="039"
cnvarr[2,40]=b2+s1+b1+s3+b1+s3
cnvarr[1,41]="040"
cnvarr[2,41]=b2+s3+b1+s1+b1+s3
cnvarr[1,42]="041"
cnvarr[2,42]=b2+s3+b1+s3+b1+s1
cnvarr[1,43]="042"
cnvarr[2,43]=b1+s1+b2+s1+b3+s3
cnvarr[1,44]="043"
cnvarr[2,44]=b1+s1+b2+s3+b3+s1
cnvarr[1,45]="044"
cnvarr[2,45]=b1+s3+b2+s1+b3+s1
cnvarr[1,46]="045"
cnvarr[2,46]=b1+s1+b3+s1+b2+s3
cnvarr[1,47]="046"
cnvarr[2,47]=b1+s1+b3+s3+b2+s1
cnvarr[1,48]="047"
cnvarr[2,48]=b1+s3+b3+s1+b2+s1
cnvarr[1,49]="048"
cnvarr[2,49]=b3+s1+b3+s1+b2+s1
cnvarr[1,50]="049"
cnvarr[2,50]=b2+s1+b1+s3+b3+s1
cnvarr[1,51]="050"
cnvarr[2,51]=b2+s3+b1+s1+b3+s1
cnvarr[1,52]="051"
cnvarr[2,52]=b2+s1+b3+s1+b1+s3
cnvarr[1,53]="052"
cnvarr[2,53]=b2+s1+b3+s3+b1+s1
cnvarr[1,54]="053"
cnvarr[2,54]=b2+s1+b3+s1+b3+s1
cnvarr[1,55]="054"
cnvarr[2,55]=b3+s1+b1+s1+b2+s3
cnvarr[1,56]="055"
cnvarr[2,56]=b3+s1+b1+s3+b2+s1
cnvarr[1,57]="056"
cnvarr[2,57]=b3+s3+b1+s1+b2+s1
cnvarr[1,58]="057"
cnvarr[2,58]=b3+s1+b2+s1+b1+s3
cnvarr[1,59]="058"
cnvarr[2,59]=b3+s1+b2+s3+b1+s1
cnvarr[1,60]="059"
cnvarr[2,60]=b3+s3+b2+s1+b1+s1
cnvarr[1,61]="060"
cnvarr[2,61]=b3+s1+b4+s1+b1+s1
cnvarr[1,62]="061"
cnvarr[2,62]=b2+s2+b1+s4+b1+s1
cnvarr[1,63]="062"
cnvarr[2,63]=b4+s3+b1+s1+b1+s1
cnvarr[1,64]="063"
cnvarr[2,64]=b1+s1+b1+s2+b2+s4
cnvarr[1,65]="064"
cnvarr[2,65]=b1+s1+b1+s4+b2+s2
cnvarr[1,66]="065"
cnvarr[2,66]=b1+s2+b1+s1+b2+s4
cnvarr[1,67]="066"
cnvarr[2,67]=b1+s2+b1+s4+b2+s1
cnvarr[1,68]="067"
cnvarr[2,68]=b1+s4+b1+s1+b2+s2
cnvarr[1,69]="068"
cnvarr[2,69]=b1+s4+b1+s2+b2+s1
cnvarr[1,70]="069"
cnvarr[2,70]=b1+s1+b2+s2+b1+s4
cnvarr[1,71]="070"
cnvarr[2,71]=b1+s1+b2+s4+b1+s2
cnvarr[1,72]="071"
cnvarr[2,72]=b1+s2+b2+s1+b1+s4
cnvarr[1,73]="072"
cnvarr[2,73]=b1+s2+b2+s4+b1+s1
cnvarr[1,74]="073"
cnvarr[2,74]=b1+s4+b2+s1+b1+s2
cnvarr[1,75]="074"
cnvarr[2,75]=b1+s4+b2+s2+b1+s1
cnvarr[1,76]="075"
cnvarr[2,76]=b2+s4+b1+s2+b1+s1
cnvarr[1,77]="076"
cnvarr[2,77]=b2+s2+b1+s1+b1+s4
cnvarr[1,78]="077"
cnvarr[2,78]=b4+s1+b3+s1+b1+s1
cnvarr[1,79]="078"
cnvarr[2,79]=b2+s4+b1+s1+b1+s2
cnvarr[1,80]="079"
cnvarr[2,80]=b1+s3+b4+s1+b1+s1
cnvarr[1,81]="080"
cnvarr[2,81]=b1+s1+b1+s2+b4+s2
cnvarr[1,82]="081"
cnvarr[2,82]=b1+s2+b1+s1+b4+s2
cnvarr[1,83]="082"
cnvarr[2,83]=b1+s2+b1+s2+b4+s1
cnvarr[1,84]="083"
cnvarr[2,84]=b1+s1+b4+s2+b1+s2
cnvarr[1,85]="084"
cnvarr[2,85]=b1+s2+b4+s1+b1+s2
cnvarr[1,86]="085"
cnvarr[2,86]=b1+s2+b4+s2+b1+s1
cnvarr[1,87]="086"
cnvarr[2,87]=b4+s1+b1+s2+b1+s2
cnvarr[1,88]="087"
cnvarr[2,88]=b4+s2+b1+s1+b1+s2
cnvarr[1,89]="088"
cnvarr[2,89]=b4+s2+b1+s2+b1+s1
cnvarr[1,90]="089"
cnvarr[2,90]=b2+s1+b2+s1+b4+s1
cnvarr[1,91]="090"
cnvarr[2,91]=b2+s1+b4+s1+b2+s1
cnvarr[1,92]="091"
cnvarr[2,92]=b4+s1+b2+s1+b2+s1
cnvarr[1,93]="092"
cnvarr[2,93]=b1+s1+b1+s1+b4+s3
cnvarr[1,94]="093"
cnvarr[2,94]=b1+s1+b1+s3+b4+s1
cnvarr[1,95]="094"
cnvarr[2,95]=b1+s3+b1+s1+b4+s1
cnvarr[1,96]="095"
cnvarr[2,96]=b1+s1+b4+s1+b1+s3
cnvarr[1,97]="096"
cnvarr[2,97]=b1+s1+b4+s3+b1+s1
cnvarr[1,98]="097"
cnvarr[2,98]=b4+s1+b1+s1+b1+s3
cnvarr[1,99]="098"
cnvarr[2,99]=b4+s1+b1+s3+b1+s1
cnvarr[1,100]="099"
cnvarr[2,100]=b1+s1+b3+s1+b4+s1
cnvarr[1,101]="100"
cnvarr[2,101]=b1+s1+b4+s1+b3+s1
cnvarr[1,102]="101"
cnvarr[2,102]=b3+s1+b1+s1+b4+s1
cnvarr[1,103]="102"
cnvarr[2,103]=b4+s1+b1+s1+b3+s1
cnvarr[1,104]="103"
cnvarr[2,104]=b2+s1+b1+s4+b1+s2
cnvarr[1,105]="104"
cnvarr[2,105]=b2+s1+b1+s2+b1+s4
cnvarr[1,106]="105"
cnvarr[2,106]=b2+s1+b1+s2+b3+s2

for i = 1 to len(m) step 3
  j = j + 1
  cstr = substr(m,i,3)
  rtnc = asubscript(cnvarr,ascan(cnvarr,cstr,aelement(cnvarr,1,1),106),2)
  if rtnc > 0
    rtnstr = rtnstr + cnvarr[2,rtnc]
    chktot = chktot + iif(j=1,val(cstr),(j-1)*val(cstr))
  endif
endfor

* calculate the check digit
rtnc = asubscript(cnvarr,ascan(cnvarr,padl(ltrim(str(mod(chktot,103),3,0)),3,"0"),;
             aelement(cnvarr,1,1),106),2)

if rtnc > 0
  * add check digit and stop code
  rtnstr = rtnstr + cnvarr[2,rtnc] + b2 + s3 + b3 + s1 + b1 + s1 + b2
endif

return rtnstr

*********************************************************************

*********************************************************************
* UPC Code Section
*********************************************************************

Procedure BC_UPCheck
Parameter msg

* Calculate and add the check character *
bcc = 0
bco = 0
for i = 1 to len(msg)
  if i / 2 = int(i / 2)
    bcc = bcc + val(substr(msg, i, 1))
  else
    bco = bco + val(substr(msg, i, 1))
  endif
endfor

bco = bco * 3 + bcc
bcc = int((bco+9) / 10) * 10
bcc = bcc - bco

*** Finish the UPC symbol.  Add the checksum character
msg = msg + str(bcc, 1)
return msg

*********************************************************************

Procedure GBarUPCA
Parameters BC_GMode,pval,codestr

codestr = ""
BC_GCols = 0

* Print Guard Bars only
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,42,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,42,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)

return BC_GCols

*********************************************************************

Procedure CCodeUPCA
Parameters BC_GMode,pval,codestr

codestr = ""
BC_GCOls = 0

* Left guard bars
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)

* Print left block
for k = 1 TO 24
  do case
    case substr(BC_CStyle[21],k,1) = chr(1)
      null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(2)
      null=MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(3)
      null=MkCodeStr(@codestr,.f.,3,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(4)
      null=MkCodeStr(@codestr,.f.,4,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(5)
      null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(6)
      null=MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(7)
      null=MkCodeStr(@codestr,.t.,3,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(8)
      null=MkCodeStr(@codestr,.t.,4,@BC_GCols,pval)
  endcase
endfor

* Print middle Guard Bars
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)

* Print right block
for k = 25 TO 48
  do case
    case substr(BC_CStyle[21],k,1) = chr(5)
      null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(6)
      null=MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(7)
      null=MkCodeStr(@codestr,.f.,3,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(8)
      null=MkCodeStr(@codestr,.f.,4,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(1)
      null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(2)
      null=MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(3)
      null=MkCodeStr(@codestr,.t.,3,@BC_GCols,pval)
    case substr(BC_CStyle[21],k,1) = chr(4)
      null=MkCodeStr(@codestr,.t.,4,@BC_GCols,pval)
  endcase
endfor

* Print right Guard Bars
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)

return BC_GCols

*********************************************************************

Procedure GBarUPCE
Parameters BC_GMode,pval,codestr

codestr = ""
BC_GCols = 0

null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,42,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)

return BC_GCols

*********************************************************************

Procedure CCodeUPCE
Parameters BC_GMode,pval,codestr

codestr = ""
BC_GCOls = 0

* Left guard bars
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)

* Print left block
m = 1
for j = 1 TO 6
  if j = 3 .or. j = 4 .or. j = 6
    for k = 1 to 4
      do case
        case substr(BC_CStyle[21],m,1) = chr(1)
          null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(2)
          null=MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(3)
          null=MkCodeStr(@codestr,.f.,3,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(4)
          null=MkCodeStr(@codestr,.f.,4,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(5)
          null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(6)
          null=MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(7)
          null=MkCodeStr(@codestr,.t.,3,@BC_GCols,pval)
          case substr(BC_CStyle[21],m,1) = chr(8)
          null=MkCodeStr(@codestr,.t.,4,@BC_GCols,pval)
        endcase
        m = m + 1
      endfor
   else
     l = m + 3
     for k = 1 to 4
        do case
          case substr(BC_CStyle[21],l,1) = chr(5)
          null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(6)
          null=MkCodeStr(@codestr,.f.,2,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(7)
          null=MkCodeStr(@codestr,.f.,3,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(8)
          null=MkCodeStr(@codestr,.f.,4,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(1)
          null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(2)
          null=MkCodeStr(@codestr,.t.,2,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(3)
          null=MkCodeStr(@codestr,.t.,3,@BC_GCols,pval)
          case substr(BC_CStyle[21],l,1) = chr(4)
          null=MkCodeStr(@codestr,.t.,4,@BC_GCols,pval)
        endcase
        m = m + 1
        l = l - 1
      endfor
   endif
  endfor

* Print right Guard Bars
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.f.,1,@BC_GCols,pval)
null=MkCodeStr(@codestr,.t.,1,@BC_GCols,pval)

return BC_GCols

*********************************************************************

Procedure BC_LPUPC
Parameters m
private t,p,os,ts,hs,fs,ob,tb,hb,fb

* Generates bar pattern for UPC Version A bar codes
*    m  = text to decode into a bit pattern
*
* Returns:
*    Bar pattern for UPC vA bar code.
*    If an illegal value is found then a null will
*    be returned to the user.

*** if the parameter is not a character then quit
if type('m')<>"C"
  return ""
endif

*** define spaces/bars
os = chr(1)
ts = chr(2)
hs = chr(3)
fs = chr(4)
ob = chr(5)
tb = chr(6)
hb = chr(7)
fb = chr(8)


*** encode message with odd parity
p=""

FOR x = 1 TO len(m)
  t = substr(m, x, 1)

  do case
    CASE t = "0"
      p = p + hs + tb + os + ob
    CASE t = "1"
      p = p + ts + tb + ts + ob
    CASE t = "2"
      p = p + ts + ob + ts + tb
    CASE t = "3"
      p = p + os + fb + os + ob
    CASE t = "4"
      p = p + os + ob + hs + tb
    CASE t = "5"
      p = p + os + tb + hs + ob
    CASE t = "6"
      p = p + os + ob + os + fb
    CASE t = "7"
      p = p + os + hb + os + tb
    CASE t = "8"
      p = p + os + tb + os + hb
    CASE t = "9"
      p = p + hs + ob + os + tb
    otherwise
      *** Invalid code found
      return ""
  endcase

  *** Each bar code has a narrow space after it
endfor

*** Return the bit pattern
return p

*********************************************************************

*********************************************************************
* PostNet Code Section
*********************************************************************

Procedure BC_PNCheck
Parameter msg

* Calculate and add the check character *
bcc = 0
for i = 1 to len(msg)
  bcc = bcc + val(substr(msg, i, 1))
endfor

bcc = iif(bcc%10 = 0,0,10 - (bcc % 10))

*** Finish the UPC symbol.  Add the checksum character
msg = msg + str(bcc, 1)
return msg

*********************************************************************

Procedure CCodePNet
Parameters BC_GMode,codestr

codestr = ""
BC_GCOls = 0

null=MkPostStr(@codestr,1,1,@BC_GCols)
null=MkPostStr(@codestr,0,1,@BC_GCols)

for k = 1 TO len(BC_CStyle[21])
  do case
    case substr(BC_CStyle[21],k,1) = chr(0)
      null=MkPostStr(@codestr,2,1,@BC_GCols)
      null=MkPostStr(@codestr,0,1,@BC_GCols)
    case substr(BC_CStyle[21],k,1) = chr(1)
      null=MkPostStr(@codestr,1,1,@BC_GCols)
      null=MkPostStr(@codestr,0,1,@BC_GCols)
  endcase
endfor

null=MkPostStr(@codestr,1,1,@BC_GCols)

return BC_GCols

*********************************************************************

Procedure BC_LPPostNet
Parameters m
private sh,lb,t,x,p

* Generates bar pattern for PostNet bar codes
*    m  = text to decode into a bit pattern
*
* Returns:
*    Bar pattern for PostNet bar code.
*    If an illegal value is found then a null will
*    be returned to the user.

*** if the parameter is not a character then quit
if type('m')<>"C"
  return ""
endif

*** define narrow/wide spaces/bars
sb = chr(0)
lb = chr(1)

*** encode message into bar code bit pattern and place
*** it into the variable
p=""

FOR x = 1 TO len(m)
  t = substr(m, x, 1)

  do case
    CASE t = "0"
      p = p + lb + lb + sb + sb + sb
    CASE t = "1"
      p = p + sb + sb + sb + lb + lb
    CASE t = "2"
      p = p + sb + sb + lb + sb + lb
    CASE t = "3"
      p = p + sb + sb + lb + lb + sb
    CASE t = "4"
      p = p + sb + lb + sb + sb + lb
    CASE t = "5"
      p = p + sb + lb + sb + lb + sb
    CASE t = "6"
      p = p + sb + lb + lb + sb + sb
    CASE t = "7"
      p = p + lb + sb + sb + sb + lb
    CASE t = "8"
      p = p + lb + sb + sb + lb + sb
    CASE t = "9"
      p = p + lb + sb + lb + sb + sb
    otherwise
      *** Invalid code found
      return ""
  endcase
endfor

*** Return the bit pattern
return p

*********************************************************************

*********************************************************************
* HP Laser Printer Driver Code Section
*********************************************************************

Procedure BC_HPPBC

uccmove = .117
* If not using relative positioning set the current cursor location
if BC_CStyle[29] = 1
  null = LP_RCurPos(BC_CStyle[3],BC_CStyle[2])
else
  null = LP_CurPos(BC_CStyle[3],BC_CStyle[2])
endif

do case
case BC_CStyle[1] = "CODE39"

  if "RIGHT" = BC_CStyle[6]
    null = LP_RCurPos(0.0,(-1) * ((len(BC_Messg)+2) / BC_CStyle[7]))
  endif

  * Save current position to print the text
  if BC_CStyle[11] = 1
    null = LP_SRCurPos(.t.)
  endif

  do case
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
      null = LP_RCurPos(BC_CStyle[8],0)
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
      null = LP_RCurPos(BC_CStyle[8] - ;
             iif(BC_CStyle[11] = 1,BC_CStyle[14] + BC_CStyle[15],0),0)
    case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] <> "ABOVE" .and. BC_CStyle[11] = 1
      null = LP_RCurPos((-1) * (BC_CStyle[14] + BC_CStyle[15]),0)
  endcase

  for i = 1 TO len(BC_CStyle[21])
     do case
       case substr(BC_CStyle[21],i,1) = chr(0)
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(1)
         null = LP_RCurPos(0.0,(2.5 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(2)
         null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(3)
         null = BC_PBox(BC_CStyle[22],(2.5 *BC_CStyle[20]))
         null = LP_RCurPos(0.0,(2.5 * BC_CStyle[20]))
    endcase
  endfor
case BC_CStyle[1] = "INTERLEAVED25"

  * set position justification
   if "RIGHT" = BC_CStyle[6]
     null = LP_RCurPos(0.0,(-1) * ((len(BC_Messg)+1) / BC_CStyle[7]))
  endif

  * Save current position to print the text
  if BC_CStyle[11] = 1
    null = LP_SRCurPos(.t.)
  endif

  do case
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
      null = LP_RCurPos(BC_CStyle[8],0)
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
      null = LP_RCurPos(BC_CStyle[8] - ;
             iif(BC_CStyle[11] = 1,BC_CStyle[14] + BC_CStyle[15],0),0)
    case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] <> "ABOVE" .and. BC_CStyle[11] = 1
      null = LP_RCurPos((-1) * (BC_CStyle[14] + BC_CStyle[15]),0)
  endcase

  for i = 1 TO len(BC_CStyle[21])
     do case
       case substr(BC_CStyle[21],i,1) = chr(0)
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(1)
         null = LP_RCurPos(0.0,(2.5 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(2)
         null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(3)
         null = BC_PBox(BC_CStyle[22],(2.5 *BC_CStyle[20]))
         null = LP_RCurPos(0.0,(2.5 * BC_CStyle[20]))
    endcase
  endfor
case BC_CStyle[1] = "CODE128"

  * set position justification

  if "RIGHT" = BC_CStyle[6]
    null = LP_RCurPos(0.0,(-1) * ((len(BC_Messg)+2) / BC_CStyle[7]))
  endif

  * Save current position to print the text
  if BC_CStyle[11] = 1
    null = LP_SRCurPos(.t.)
  endif

  do case
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
      null = LP_RCurPos(BC_CStyle[8],0)
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
      null = LP_RCurPos(BC_CStyle[8] - ;
             iif(BC_CStyle[11] = 1,BC_CStyle[14] + BC_CStyle[15],0),0)
    case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] <> "ABOVE" .and. BC_CStyle[11] = 1
      null = LP_RCurPos((-1) * (BC_CStyle[14] + BC_CStyle[15]),0)
  endcase

  for i = 1 TO len(BC_CStyle[21])
     do case
       case substr(BC_CStyle[21],i,1) = chr(0)
         null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(1)
         null = BC_PBox(BC_CStyle[22],(2 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(2)
         null = BC_PBox(BC_CStyle[22],(3 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(3)
         null = BC_PBox(BC_CStyle[22],(4 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(4)
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(5)
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(6)
         null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(7)
         null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
    endcase
  endfor
case BC_CStyle[1] = "UPC_A"

  * set position justification

  if "RIGHT" = BC_CStyle[6]
    null = LP_RCurPos(0.0,(-1) * (BC_CStyle[7]))
  endif

  * Save current position to print the text
  if BC_CStyle[11] = 1
    null = LP_SRCurPos(.t.)
  endif

  * if UCC Code standard is set move the barcode to the right the uccmove
  * distance to make room for the text.
  if BC_CStyle[36] = 1
    null = LP_RCurPos(0.0,uccmove)
  endif

  do case
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
      null = LP_RCurPos(BC_CStyle[8],0)
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
      null = LP_RCurPos(BC_CStyle[8] - ;
             iif(BC_CStyle[11] = 1,BC_CStyle[14] + BC_CStyle[15],0),0)
    case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] <> "ABOVE" .and. BC_CStyle[11] = 1
      null = LP_RCurPos((-1) * (BC_CStyle[14] + BC_CStyle[15]),0)
  endcase

  * Print inital Guard Bars
  BC_GB_ADD = BC_CStyle[22] * .05
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,BC_CStyle[20])

  * Print left block
  for i = 1 TO 24
     do case
       case substr(BC_CStyle[21],i,1) = chr(1)
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(2)
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(3)
         null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(4)
         null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(5)
         null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(6)
         null = BC_PBox(BC_CStyle[22],(2 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(7)
         null = BC_PBox(BC_CStyle[22],(3 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(8)
         null = BC_PBox(BC_CStyle[22],(4 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
    endcase
  endfor

  * Print middle Guard Bars
  null = LP_RCurPos(0.0,BC_CStyle[20])
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))

  * Print right block
  for i = 25 TO 48
     do case
       case substr(BC_CStyle[21],i,1) = chr(5)
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(6)
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(7)
         BC_CStyle[2] = BC_CStyle[2] + (3 * BC_CStyle[20])
         null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(8)
         BC_CStyle[2] = BC_CStyle[2] + (4 * BC_CStyle[20])
         null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(1)
         null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
         null = LP_RCurPos(0.0,BC_CStyle[20])
       case substr(BC_CStyle[21],i,1) = chr(2)
         null = BC_PBox(BC_CStyle[22],(2 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(3)
         null = BC_PBox(BC_CStyle[22],(3 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(4)
         null = BC_PBox(BC_CStyle[22],(4 * BC_CStyle[20]))
         null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
    endcase
  endfor

  * Print right Guard Bars
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)

case BC_CStyle[1] = "UPC_E"

  * set position justification

  if "RIGHT" = BC_CStyle[6]
    null = LP_RCurPos(0.0,(-1) * (BC_CStyle[7]))
  endif

  * Save current position to print the text
  if BC_CStyle[11] = 1
    null = LP_SRCurPos(.t.)
  endif

  do case
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
      null = LP_RCurPos(BC_CStyle[8],0)
    case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
      null = LP_RCurPos(BC_CStyle[8] - ;
             iif(BC_CStyle[11] = 1,BC_CStyle[14] + BC_CStyle[15],0),0)
    case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] <> "ABOVE" .and. BC_CStyle[11] = 1
      null = LP_RCurPos((-1) * (BC_CStyle[14] + BC_CStyle[15]),0)
  endcase

  * Print inital Guard Bars
  BC_GB_ADD = BC_CStyle[22] * .05
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,BC_CStyle[20])

  * Print left block
  i = 1
  for j = 1 TO 6
    if j = 3 .or. j = 4 .or. j = 6
      for k = 1 to 4
        do case
          case substr(BC_CStyle[21],i,1) = chr(1)
            null = LP_RCurPos(0.0,BC_CStyle[20])
          case substr(BC_CStyle[21],i,1) = chr(2)
            null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
          case substr(BC_CStyle[21],i,1) = chr(3)
            BC_CStyle[2] = BC_CStyle[2] + (3 * BC_CStyle[20])
            null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
          case substr(BC_CStyle[21],i,1) = chr(4)
            null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
          case substr(BC_CStyle[21],i,1) = chr(5)
            null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
            null = LP_RCurPos(0.0,BC_CStyle[20])
          case substr(BC_CStyle[21],i,1) = chr(6)
            null = BC_PBox(BC_CStyle[22],(2 * BC_CStyle[20]))
            null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
          case substr(BC_CStyle[21],i,1) = chr(7)
            null = BC_PBox(BC_CStyle[22],(3 * BC_CStyle[20]))
            null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
          case substr(BC_CStyle[21],i,1) = chr(8)
            null = BC_PBox(BC_CStyle[22],(4 * BC_CStyle[20]))
            null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
        endcase
        i = i + 1
      endfor
   else
     l = i + 3
     for k = 1 to 4
        do case
          case substr(BC_CStyle[21],l,1) = chr(5)
            null = LP_RCurPos(0.0,BC_CStyle[20])
          case substr(BC_CStyle[21],l,1) = chr(6)
            null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
          case substr(BC_CStyle[21],l,1) = chr(7)
            null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
          case substr(BC_CStyle[21],l,1) = chr(8)
            null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
          case substr(BC_CStyle[21],l,1) = chr(1)
            null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
            null = LP_RCurPos(0.0,BC_CStyle[20])
          case substr(BC_CStyle[21],l,1) = chr(2)
            null = BC_PBox(BC_CStyle[22],(2 * BC_CStyle[20]))
            null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
          case substr(BC_CStyle[21],l,1) = chr(3)
            null = BC_PBox(BC_CStyle[22],(3 * BC_CStyle[20]))
            null = LP_RCurPos(0.0,(3 * BC_CStyle[20]))
          case substr(BC_CStyle[21],l,1) = chr(4)
            null = BC_PBox(BC_CStyle[22],(4 * BC_CStyle[20]))
            null = LP_RCurPos(0.0,(4 * BC_CStyle[20]))
        endcase
        i = i + 1
        l = l - 1
      endfor
   endif
  endfor

  * Print right Guard Bars
  null = LP_RCurPos(0.0,BC_CStyle[20])
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
  null = BC_PBox(BC_CStyle[22],BC_CStyle[20],BC_GB_ADD)

case BC_CStyle[1] = "POSTNET"

  * set position justification

  if "RIGHT" = BC_CStyle[6]
    null = LP_RCurPos(0.0,(-1) * (BC_CStyle[20] * ((len(BC_Messg) * 10) + 3)))
  endif

  * Save current position to print the text
  if BC_CStyle[11] = 1
    null = LP_SRCurPos(.t.)
  endif

  if BC_CStyle[5] = "TOP"
      null=LP_RCurPos(BC_CStyle[8],0)
  endif

  null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
  null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))

  for i = 1 TO len(BC_CStyle[21])
     do case
       case substr(BC_CStyle[21],i,1) = chr(0)
         null = BC_PBox(BC_CStyle[22]/2,BC_CStyle[20])
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
       case substr(BC_CStyle[21],i,1) = chr(1)
         null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
         null = LP_RCurPos(0.0,(2 * BC_CStyle[20]))
    endcase
  endfor

  null = BC_PBox(BC_CStyle[22],BC_CStyle[20])
endcase

* Restore current cursor position
if BC_CStyle[11] = 1
  null = LP_SRCurPos(.f.)
endif

Return ""

*********************************************************************

Procedure BC_PBox
Parameters BC_H,BC_B,BC_GD

* if it is printing a Guard band add the guard distance to the height
BC_H = iif(type("BC_GD") = "N",BC_H + BC_GD, BC_H)

* save cursor position
  null = LP_SRCurPos(.t.)

* get negative height value
BC_NH = (-1) * BC_H

* reposition cursor to top left corner
null = LP_RCurPos(BC_NH,0.0)

* if printing guard band add gb distance again
BC_H = iif(type("BC_GD") = "N",BC_H + BC_GD, BC_H)
null = LP_RBoxShade(BC_H,BC_B,100)

* restore cursor position
  null = LP_SRCurPos(.f.)

return ""

*********************************************************************

Procedure BC_HPPTXT

* LaserPrinter Specific
if Post_LJ_
  null = L_FontPick("COURIER",12)
  BC_Cl = .10
  BC_Ch = .10
else
  null = P_FontPick("HEN",12)
  BC_Cl = .10
  BC_Ch = .10
endif

do case
  case BC_CStyle[1] = "CODE39"
    do case
      case BC_CStyle[13] = "CENTER" .or. BC_CStyle[13] = "SPLIT"
        BC_Tx = (((len(BC_Messg) * 12 * BC_CStyle[20]) + (BC_Cstyle[20] * 36))/2);
                 - (BC_Cl * len(BC_Messg))/2
      case BC_CStyle[13] = "RIGHT"
        BC_Tx = ((len(BC_Messg) * 12 * BC_CStyle[20]) + (BC_Cstyle[20] * 36));
                 - (BC_Cl * len(BC_Messg))
      otherwise
        BC_Tx = 0
    endcase
    * set text position justification
    do case
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (BC_CStyle[14])
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
        BC_Ty = BC_CStyle[8]
      case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (-1) * (BC_CStyle[8] - BC_CStyle[14])
      otherwise
        BC_Ty = 0
    endcase
    null = LP_RCurPos(BC_Ty,BC_Tx)
    null = LP_RSayIt(BC_Messg)
  case BC_CStyle[1] = "INTERLEAVED25"
    do case
      case BC_CStyle[13] = "CENTER" .or. BC_CStyle[13] = "SPLIT"
        BC_Tx = (((len(BC_Messg) * 8 * BC_CStyle[20]) + (BC_Cstyle[20] * 9))/2);
                 - (BC_Cl * len(BC_Messg))/2
      case BC_CStyle[13] = "RIGHT"
        BC_Tx = ((len(BC_Messg) * 8 * BC_CStyle[20]) + (BC_Cstyle[20] * 9));
                 - (BC_Cl * len(BC_Messg))
      otherwise
        BC_Tx = 0
    endcase
    * set text position justification
    do case
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (BC_CStyle[14])
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
        BC_Ty = BC_CStyle[8]
      case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (-1) * (BC_CStyle[8] - BC_CStyle[14])
      otherwise
        BC_Ty = 0
    endcase
    null = LP_RCurPos(BC_Ty,BC_Tx)
    null = LP_RSayIt(BC_Messg)
  case BC_CStyle[1] = "CODE128"
    * calculate the length of the barcode
    cont = .t.
    i = 1
    m128len = 0
    m128tlen = 0
    bandc = .f.
    do while cont
      t = substr(BC_Messg,i,2)
      if t = chr(128) + chr(128)
        if substr(BC_Messg,i+2,1) = chr(3) .or. substr(BC_Messg,i+2,1) = chr(9)
          bandc = .t.
        else
          if substr(BC_Messg,i+2,1) = chr(4) .or. substr(BC_Messg,i+2,1) = chr(5)
            bandc = .f.
          endif
        endif
        m128len = m128len + 1
        i = i + 3
      else
        m128len = m128len + 1
        m128tlen = m128tlen + iif(bandc,2,1)
        i = i + iif(bandc,2,1)
      endif
      if i+2 > len(BC_Messg)
        m128len = m128len + iif(bandc,1,(len(BC_Messg) - i + 1))
        m128tlen = m128tlen + iif(bandc,2,(len(BC_Messg) - i + 1))
        cont = .f.
      endif
    enddo

    do case
      case BC_CStyle[13] = "CENTER" .or. BC_CStyle[13] = "SPLIT"
        BC_Tx = (((m128len * 11 * BC_CStyle[20]) + (BC_Cstyle[20] * 24))/2);
                 - (BC_Cl * m128tlen)/2
      case BC_CStyle[13] = "RIGHT"
        BC_Tx = ((m128len * 11 * BC_CStyle[20]) + (BC_Cstyle[20] * 24));
                 - (BC_Cl * m128tlen)
      otherwise
        BC_Tx = 0
    endcase
    * set text position justification
    do case
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (BC_CStyle[14])
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
        BC_Ty = BC_CStyle[8]
      case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (-1) * (BC_CStyle[8] - BC_CStyle[14])
      otherwise
        BC_Ty = 0
    endcase
    null = LP_RCurPos(BC_Ty,BC_Tx)
    null = LP_RSayIt(BC_Messg)
  case BC_CStyle[1] = "UPC_A" .and. BC_CStyle[36] = 0
    BC_CStyle[20] = BC_CStyle[7]/95
    do case
      case BC_CStyle[13] = "SPLIT"
        BC_Tx1 = (BC_CStyle[20] * 24) - (BC_Cl * len(substr(BC_Messg,1,6)))/2
        BC_Tx2 = (BC_CStyle[20] * 71) - (BC_Cl * len(substr(BC_Messg,7,6)))/2
      otherwise
        BC_Tx1 = (BC_CStyle[20] * 48) - (BC_Cl * len(BC_Messg))/2
    endcase
    * set text position justification
    do case
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = BC_CStyle[14]
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
        BC_Ty = BC_CStyle[8]
      case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (-1) * (BC_CStyle[8] - BC_CStyle[14])
      otherwise
        BC_Ty = 0
    endcase
    do case
      case BC_CStyle[13] = "SPLIT"
        * save cursor position
        null = LP_SRCurPos(.t.)
        null = LP_RCurPos(BC_Ty,BC_Tx1)
        null = LP_RSayIt(substr(BC_Messg,1,6))
        * restore cursor position
        null = LP_SRCurPos(.f.)
        null = LP_RCurPos(BC_Ty,BC_Tx2)
        null = LP_RSayIt(substr(BC_Messg,7,6))
      otherwise
        null = LP_RCurPos(BC_Ty,BC_Tx1)
        null = LP_RSayIt(BC_Messg)
    endcase
  case BC_CStyle[1] = "UPC_A" .and. BC_CStyle[36] = 1
    BC_Tx1 = (BC_CStyle[20] * 24) - (BC_Cl * 5)/2
    BC_Tx2 = (BC_CStyle[20] * 71) - (BC_Cl * 5)/2
    BC_Tx3 = 0
    BC_Tx4 = BC_CStyle[7]
    * set text position justification
    do case
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = BC_CStyle[14]
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
        BC_Ty = BC_CStyle[8]
      case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (-1) * (BC_CStyle[8] - BC_CStyle[14])
      otherwise
        BC_Ty = 0
    endcase
    if Post_LJ_
      null = L_FontAtt("PITCH_17")
    else
      null = P_FontPick("HEN",8)
    endif
    null = LP_SRCurPos(.t.)
    null = LP_RCurPos(BC_Ty,BC_Tx3)
    null = LP_RSayIt(substr(BC_Messg,1,1))
    null = LP_SRCurPos(.f.)
    if Post_LJ_
      null = L_FontAtt("PITCH_10")
    else
      null = P_FontPick("HEN",12)
    endif
    * save cursor position
    null = LP_SRCurPos(.t.)
    null = LP_RCurPos(BC_Ty,BC_Tx1+.117)
    null = LP_RSayIt(substr(BC_Messg,2,5))
    * restore cursor position
    null = LP_SRCurPos(.f.)
    null = LP_SRCurPos(.t.)
    null = LP_RCurPos(BC_Ty,BC_Tx2+.117)
    null = LP_RSayIt(substr(BC_Messg,7,5))
    null = LP_SRCurPos(.f.)
    if Post_LJ_
      null = L_FontAtt("PITCH_17")
    else
      null = P_FontPick("HEN",8)
    endif
    null = LP_SRCurPos(.t.)
    null = LP_RCurPos(BC_Ty,BC_Tx4+.058+.117)
    null = LP_RSayIt(substr(BC_Messg,12,1))
    null = LP_SRCurPos(.f.)
    if Post_LJ_
      null = L_FontAtt("PITCH_10")
    else
      null = P_FontPick("HEN",12)
    endif
  case BC_CStyle[1] = "UPC_E"
    BC_CStyle[20] = BC_CStyle[7]/51
    BC_Tx = (BC_CStyle[20] * 24) - (BC_Cl * len(BC_Messg))/2
    * set text position justification
    do case
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = BC_CStyle[14]
      case BC_CStyle[5] = "TOP" .and. BC_CStyle[12] <> "ABOVE"
        BC_Ty = BC_CStyle[8]
      case BC_CStyle[5] <> "TOP" .and. BC_CStyle[12] = "ABOVE"
        BC_Ty = (-1) * (BC_CStyle[8] - BC_CStyle[14])
      otherwise
        BC_Ty = 0
    endcase
    null = LP_RCurPos(BC_Ty,BC_Tx)
    null = LP_RSayIt(BC_Messg)
endcase

return ""

*********************************************************************

Procedure LP_SetPage
Parameters PL_, Ori_, FP_, FName_, Layout_, PGS_, Port_
 * Post_LJ_ - If .t. work in LaserJet Mode else in PostScript Mode
 * LP_FP_ - If .t. work sent to Printer else to file or
 *
 * LP_FP_ - If 1 Work sent to Printer
 *             2 Work sent to file
 *             3 Work return to calling procedure as string
 * FName_ - File name if work is sent to file
 * LP_Layout_ - 1 Work from Top of Page in Inches
 *              2 Work from Top of Page in Points (72 points/inch)
 *              3 Work from Bottom of Page in Inches
 *              4 Work from Bottom of Page in Points
 * Always work from left side of page.
 * PG_Size_ - Total page length in proper format ( default 11" or 720pts)
 * Port_ - Output port name i.e. LPT1, LPT2, etc.

Public Post_LJ_, LP_FP_, LP_Layout_, PG_Size_, crlf, ESC, TXT_Size_
Public LP_PSrc, LP_Str, LP_Port
Public BC_CStyle[36]
Public TMargin, LMargin

Set Talk Off
BC_CStyle[31] = 1
LP_Str = ""
Post_LJ_ = iif(type("PL_") = "L",PL_,.t.)
FP_ = iif(!empty(FP_), FP_, 1)
FNAME_ = iif(!empty(FNAME_), FNAME_, "")


* Correct if using older syntax
LP_FP_ = iif(type("FP_")="L",iif(FP_,1,2),FP_)

* To make code readable in file
crlf = CHR(13) + CHR(10)
* Escape code for LJ
ESC  = CHR(27)

* Set Defaults of LP_Layout_ to 1 if not entered
LP_Layout_ = iif(!empty(Layout_),Layout_,1)
Ori_ = iif(!empty(Ori_),Ori_,"P")

* Set Default of PG_Size_ to 11" or 792pts if not supplied
If LP_Layout_ = 1 .or. LP_Layout_ = 3
  PG_Size_ = iif(!empty(PG_Size_), PG_Size_, 11)
Else
  PG_Size_ = iif(!empty(PG_Size_), PG_Size_, 792)
Endif

* Set default LP_PSrc to 0 (Current page setting)
* null = LP_SetPSrc(0)

* Initialize Printer or Create/Open Filename
null=LP_SetPort(iif(!empty(Port_),Port_,"LPT1")) && Set Default Port to LPT1
LP_Port = iif(!empty(Port_),Port_,"LPT1")
**** 24/1/2000 LO QUITO POR POSIBLES INCIDENCIAS CON OFICINA 2000
*null=LP_Flprint(LP_FP_,FName_)

TMargin = 0
LMargin = 0

* Initialize setup and set orientation and default font
Do Case
  Case Post_LJ_ .and. LP_FP_ <> 3
*    null=L_Orient(Ori_)
    null=L_FontAtt("COURIER")
    null=L_FontAtt("ROMAN8")
    null=L_FontAtt("FIXED")
    null=L_FontAtt("BOLD_OFF")
    null=L_FontAtt("ITALICS_OFF")
    null=L_FontAtt("UNDERLINE_OFF")
    null=L_FontAtt("PITCH_10")
    TXT_Size_ = 12

  Case !Post_LJ_ .and. LP_FP_ <> 3
    null=LP_Out("%!PS-Adobe-1.0"+crlf)            && Standard PS header info
    null=LP_Out("%%Title: PS_LIB output"+crlf)
    null=LP_Out("%%Creator: Ferret Software's PS Library"+crlf)
    null=LP_Out("%%CreationDate: " + DTOC(DATE())+crlf)
    null=LP_Out("%%EndComments" + crlf)
    null=P_Orient(Ori_)
    null=P_FontPick('HEN',12)
    TXT_Size_ = 12
EndCase
Return ""   && end SetPage

* ---------------------------------------------------------

Procedure LP_SetMargin
Parameters TopM, LeftM

If Post_LJ_
  null=LP_Out(ESC + "&l0E")   && Set the printer top margin to zero
Endif

TMargin = TopM
LMargin = LeftM

Return ""

* ---------------------------------------------------------

Procedure LP_SetPort
Parameters Port
Public LP_Port

LP_Port = Port

Return ""

* ---------------------------------------------------------

Procedure LP_Flprint
Parameters LP_FP_,FName_
Public F_PTR
Do Case
  Case LP_FP_ = 1
    set printer to &LP_Port
  Case LP_FP_ = 2
    set printer to &FName_
EndCase
Return ""   && end Flprint

* ---------------------------------------------------------

Procedure LP_Out
Parameters str
If LP_FP_ = 3
  LP_Str = LP_Str + str
Else
  ??? str
endif

Return str

* ---------------------------------------------------------

*  Calculation Routines

* ---------------------------------------------------------

Procedure V_AdjR
Parameters rval
If Post_LJ_      && LJ Printer
  Do Case
    Case LP_Layout_ = 1
      rtn_val = str((720 * (rval + TMargin)),5)  && Convert inches to decipoints
    Case LP_Layout_ = 2
      rtn_val = str((10 * (rval + TMargin)),5)  && Convert points to decipoints
    Case LP_Layout_ = 3
      rtn_val = str(720 * (PG_Size_ - rval),5)  && Convert inches to decipoints
                                               && Adjust for Page Length
    Case LP_Layout_ = 4
      rtn_val = str(10 * (PG_Size_ - rval),5)  && Convert points to decipoints
                                               && Adjust for Page Length
  Endcase
Else
  Do Case
    Case LP_Layout_ = 1
      rtn_val = str((72*(PG_Size_ - rval - TMargin)),5)
    Case LP_Layout_ = 2
      rtn_val = str((PG_Size_ - rval - TMargin),5)
    Case LP_Layout_ = 3
      rtn_val = str((72 * rval),5)
    Case LP_Layout_ = 4
      rtn_val = str(rval,5)
  Endcase
Endif
Return rtn_val

* ---------------------------------------------------------

Procedure V_AdjC
Parameters rval
If Post_LJ_
  If LP_Layout_ = 1 .or. LP_Layout_ = 3
    rtn_val = str((720 * (rval + LMargin)),5)
  Else
    rtn_val = str((10 * (rval + LMargin)),5)
  Endif
Else
  If LP_Layout_ = 1 .or. LP_Layout_ = 3
    rtn_val = str((72 * (rval + LMargin)),5)
  Else
    rtn_val = str((rval + LMargin),5)
  Endif
Endif
Return rtn_val

* ---------------------------------------------------------

Procedure V_RAdjR
Parameters rval
If Post_LJ_      && LJ Printer
  Do Case
    Case LP_Layout_ = 1
      rtn_val = str(720 * rval,5)  && Convert inches to decipoints
    Case LP_Layout_ = 2
      rtn_val = str(10 * rval,5)  && Convert points to decipoints
    Case LP_Layout_ = 3
      rtn_val = str((-1) * 720 * rval,5)  && Convert inches to decipoints
    Case LP_Layout_ = 4
      rtn_val = str((-1) * 10 * rval,5)  && Convert points to decipoints
  Endcase
Else
  Do Case
    Case LP_Layout_ = 1
      rtn_val = str((-1) * 72 * rval,5)
    Case LP_Layout_ = 2
      rtn_val = str((-1) * rval,5)
    Case LP_Layout_ = 3
      rtn_val = str(72 * rval,5)
    Case LP_Layout_ = 4
      rtn_val = str(rval,5)
  Endcase
Endif
Return rtn_val

* ---------------------------------------------------------
Procedure V_RAdjC
Parameters rval
If Post_LJ_      && LJ Printer
  Do Case
    Case LP_Layout_ = 1
      rtn_val = str(720 * rval,5)  && Convert inches to decipoints
    Case LP_Layout_ = 2
      rtn_val = str(10 * rval,5)  && Convert points to decipoints
    Case LP_Layout_ = 3
      rtn_val = str(720 * rval,5)  && Convert inches to decipoints
    Case LP_Layout_ = 4
      rtn_val = str(10 * rval,5)  && Convert points to decipoints
  Endcase
Else
  Do Case
    Case LP_Layout_ = 1
      rtn_val = str(72 * rval,5)
    Case LP_Layout_ = 2
      rtn_val = str(rval,5)
    Case LP_Layout_ = 3
      rtn_val = str(72 * rval,5)
    Case LP_Layout_ = 4
      rtn_val = str(rval,5)
  Endcase
Endif
Return rtn_val

* ---------------------------------------------------------

Procedure LP_PT_IN
Parameters rval
Return str((rval / 72),5)

* ---------------------------------------------------------

Procedure LP_IN_PT
Parameters rval
Return str((72 * rval),5)

* ---------------------------------------------------------

Procedure LP_IN_DP
Parameters rval
Return str((720 * rval),5)

* ---------------------------------------------------------

Procedure LP_PT_DP
Parameters rval
Return str((100 * rval),5)

* ---------------------------------------------------------

Procedure LP_PTIN
Parameters nval
If LP_Layout_ = 1 .or. LP_Layout_ = 3
  rtn_val = LP_IN_PT(nval)
Else
  rtn_val = LP_PT_IN(nval)
Endif
Return rtn_val

* ---------------------------------------------------------

Procedure LP_INDP
Parameters nval
If LP_Layout_ = 1 .or. LP_Layout_ = 3
  rtn_str = LP_IN_DP(nval)
Else
  rtn_str = LP_PT_DP(nval)
Endif
Return rtn_str

* ---------------------------------------------------------

* LaserJet and PostScript Routines

* ---------------------------------------------------------

Procedure LP_SayIt
Parameters row_, col_, txt_, pict_
If Post_LJ_
  null=LP_Out(L_SayIt(row_, col_, txt_, pict_))
Else
  null=LP_Out(P_SayIt(row_, col_, txt_, pict_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_RSayIt
Parameters txt_, pict_
If Post_LJ_
  null=LP_Out(L_RSayIt(txt_, pict_))
Else
  null=LP_Out(P_RSayIt(txt_, pict_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_FSayIt
Parameters f_type_, row_, col_, txt_, pict_
If Post_LJ_
  null=LP_Out(L_FSayIt(f_type_, row_, col_, txt_, pict_))
Else
  null=LP_Out(P_FSayIt(f_type_, row_, col_, txt_, pict_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_CurPos
Parameters row_, col_
If Post_LJ_
  null=LP_Out(L_CurPos(row_, col_))
Else
  null=LP_Out(P_CurPos(row_, col_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_RCurPos
Parameters row_, col_
If Post_LJ_
  null=LP_Out(L_RCurPos(row_, col_))
Else
  null=LP_Out(P_RCurPos(row_, col_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_RCPos
Parameters row_, col_
If Post_LJ_
  null=LP_Out(L_RCPos(row_, col_))
Else
  null=LP_Out(P_RCPos(row_, col_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_SRCurPos
Parameter SAVREST

If Post_LJ_
  null=LP_Out(L_SRCurPos(SAVREST))
Else
  null=LP_Out(P_SRCurPos(SAVREST))
Endif

Return ""

* ---------------------------------------------------------

Procedure LP_LineDraw
Parameters row_, col_, erow_, ecol_, thick_
If Post_LJ_
  If abs(row_ - erow_) > abs(col_ - ecol_)
    null=LP_Out(L_VLine(row_, col_, abs(row_ - erow_), thick_))
  Else
    null=LP_Out(L_HLine(row_, col_, abs(col_ - ecol_), thick_))
  Endif
Else
  null=LP_Out(P_LineDraw(row_, col_, erow_, ecol_, thick_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_BoxDraw
Parameters row_, col_, erow_, ecol_, thick_
If Post_LJ_
  null=LP_Out(L_BoxDraw(row_, erow_, col_, ecol_, thick_))
Else
  null=LP_Out(P_BoxDraw(row_, col_, abs(col_ - ecol_), abs(row_ - erow_),thick_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_RBoxDraw
Parameters row_, col_, thick_
if Post_LJ_
  null = L_RBoxDraw(row_, col_, thick_)
Else
  null = LP_Out(P_RBoxDraw(row_, col_, thick_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_RBoxDraw
Parameters row_, col_, thick_
if Post_LJ_
  null = L_RBoxDraw(row_, col_, thick_)
Else
  null = LP_Out(P_RBoxDraw(row_, col_, thick_))
Endif
 return ""
* ---------------------------------------------------------

Procedure LP_RBoxShade
Parameters row_, col_, gray_
if Post_LJ_
  null=LP_Out(L_RBoxShade(row_, col_, gray_))
Else
  null=LP_Out(P_RBoxShade(row_, col_, gray_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_BoxShade
Parameters row_, col_, erow_, ecol_, gray_
if Post_LJ_
  null=LP_Out(L_BoxShade(row_, erow_, col_, ecol_, gray_))
Else
  null=LP_Out(P_BoxShade(row_, col_, abs(col_ - ecol_), abs(row_ - erow_), gray_))
Endif
Return ""

* ---------------------------------------------------------

Procedure LP_SetPSrc
Parameters Val

If Post_LJ_
  null=L_SetPSrc(Val)   && Set the Page Source

Endif

Return ""

* ---------------------------------------------------------

Procedure LP_EndPrint

* End print job
null=LP_Eject()
If Post_LJ_
Else
  null=LP_Out("%!END" + crlf + "")      && Clears up the end
Endif

Return ""

* ---------------------------------------------------------

Procedure LP_Eject

If Post_LJ_
  null=LP_Out(ESC + "&l" + ltrim(str(LP_PSrc)) + "H")
else
  null=LP_Out("showpage"+crlf)
endif

Return ""

* ---------------------------------------------------------

Procedure L_Orient
Parameters Orient_

If "LAND" = Upper(Orient_)
  null=LP_Out(ESC + "&l1O")
Else
  null=LP_Out(ESC + "&l0O")
Endif
Return ""

* ---------------------------------------------------------

Procedure L_SetPSrc
Parameters Val

LP_PSrc = Val

Return ""
* ---------------------------------------------------------

Procedure L_CurPos
Parameters row_,col_
Private rowc_,colc_

rowc_   = V_AdjR(row_)
colc_    = V_AdjC(col_)

Return ESC + "&a" + rowc_ + "v" + colc_ + "H"

* ---------------------------------------------------------

Procedure L_RCurPos
Parameters row_,col_
Private rowc_,colc_

vpn_ = iif(row_ < 0,"","+")
hpn_ = iif(col_ < 0,"","+")
rowc_ = V_RAdjR(row_)
colc_ = V_RAdjC(col_)


Return ESC + "&a" + vpn_ + ltrim(rowc_) + "v" + hpn_ + ltrim(colc_) + "H"

* ---------------------------------------------------------

Procedure L_SRCurPos
Parameter SAVREST

* if SAVREST is .t. push current position, if .f. pop current position
if SAVREST
  temp_ = ESC + "&f0S"
else
  temp_ = ESC + "&f1S"
endif

Return temp_

* ---------------------------------------------------------

Procedure L_RCPos
Parameters row_,col_

Return ESC + "&a" + trim(str(row_)) + "R" + trim(str(col_)) + "C"

* ---------------------------------------------------------

Procedure L_Reset

null=LP_Out(ESC + "E")     && Reset printer

Return ""

* ---------------------------------------------------------

Procedure L_Clear

null=LP_Out(ESC + "*c0F")     && Clear ALL Fonts

Return ""

* ---------------------------------------------------------

Procedure L_BoxDraw
Parameters top_row,bottom_row,left_col,right_col,_thick

   _height = (bottom_row - top_row )                 && determine line lengths
   _width  = (right_col  - left_col) + (_thick)      && Adjustment for corner

   top_    = L_HLINE( top_row    , left_col   , _width  , _thick )
   left_   = L_VLINE( top_row    , left_col   , _height , _thick )
   bottom_ = L_HLINE( bottom_row , left_col   , _width  , _thick )
   right_  = L_VLINE( top_row    , right_col  , _height , _thick )

Return top_ + left_ + bottom_ + right_

* ---------------------------------------------------------

Procedure L_RBoxDraw
Parameters yval,xval,_thick

   null = L_SRCurPos(.t.)
   null = LP_Out(L_RHLINE( xval, _thick ))
   null = LP_Out(L_RVLINE( yval , _thick ))
   null = L_SRCurPos(.f.)
   null = LP_Out(L_RVLINE( yval, _thick ))
   null = LP_Out(L_RHLINE( xval, _thick ))

Return ""

* ---------------------------------------------------------

Procedure L_Copies
Parameters numcopies

null=LP_Out(ESC+"&l"+ALLTRIM(STR(num_copies))+"X")

Return ""

* ---------------------------------------------------------

Procedure L_FontLoad
Parameters font_name,_font_
Private file_hand,port_hand,font_str,file_buff

file_hand=0
port_hand=0
font_str=""
file_buff=0

null=LP_Out(ESC+"*c"+right(str(100000+_font_,6),5)+"D")

If file(font_name)

  file_hand=fopen(font_name,10)
  port_hand=fopen(LP_Port,12)
  file_buff=1024
  null=fseek(file_hand,0,0)
  set console off
  Do While !feof(file_hand)
    font_str=fread(file_hand,file_buff)
    null=fwrite(port_hand,font_str)
  Enddo
  set console on
  null=fclose(file_hand)
  null=fclose(port_hand)

Endif

null=LP_Out(ESC+"*c5F")

Return ""

* ---------------------------------------------------------

Procedure L_FontDel

string_ = ESC + "*c0F"
null=LP_Out(string_)

Return ""

* ---------------------------------------------------------

Procedure L_SFPick
Parameters font_id_, _size

TXT_Size_ = iif(!empty(_size),_size,12)

string_ = ESC + "(" + RIGHT(STR(100000+font_id_,6),5) + "X"
null=LP_Out(string_)

Return ""

* ---------------------------------------------------------

Procedure L_FontPick
Parameters typeface_, height_

TXT_Size_ = height_
Do Case
  Case Upper(typeface_) = "COURIER" .or. Upper(typeface_) = "LINEPRINTER"
    null=L_FontAtt("Fixed")
  Otherwise
    null=L_FontAtt("Proportional")
Endcase

if !empty(typeface_)
  null=L_FontAtt(typeface_)
endif

if !empty(height_)
  null=LP_Out(ESC + "(s"+ltrim(str(height_))+"v")
endif

return ""

* ---------------------------------------------------------

Procedure L_FontAtt
Parameters comm_

Do Case
* Type Face Family
  Case Upper(comm_) = "COURIER"
     ret_val= ESC+"(s3t"
  Case Upper(comm_) = "LINEPRINTER"
     ret_val= ESC+"(s0t"
  Case Upper(comm_) = "CG_TIMES"
     ret_val= ESC+"(s4101t"
  Case Upper(comm_) = "UNIVERS"
     ret_val= ESC+"(s4148t"
* Symbol Set
  Case Upper(comm_) = "ROMAN8"
     ret_val= ESC+"(8U"
  Case Upper(comm_) = "ECMA94"
     ret_val= ESC+"(0N"
  Case Upper(comm_) = "PC8"
     ret_val= ESC+"(10U"
  Case Upper(comm_) = "PC8DN"
     ret_val= ESC+"(11U"
  Case Upper(comm_) = "PC850"
     ret_val= ESC+"(12U"
  Case Upper(comm_) = "LEGAL"
     ret_val= ESC+"(1U"
* Spacing
  Case Upper(comm_) = "FIXED"
     ret_val= ESC+"(s0P"
  Case Upper(comm_) = "PROPORTIONAL"
     ret_val= ESC+"(s1P"
* Attributes
  Case Upper(comm_) = "UNDERLINE_ON"
     ret_val = ESC + "&d1D"
  Case Upper(comm_) = "UNDERLINE_OFF"
    ret_val  = ESC + "&d@"
  Case Upper(comm_) = "BOLD_ON"
    ret_val  = ESC + "(s3B"
  Case Upper(comm_) = "BOLD_OFF"
    ret_val = ESC + "(s0B"
  Case Upper(comm_) = "ITALICS_ON"
    ret_val = ESC + "(s1S"
  Case Upper(comm_) = "ITALICS_OFF"
    ret_val = ESC + "(s0S"
  Case Upper(comm_) = "PITCH_10"
   ret_val = ESC + "(s10h12v"            && Also sets height to 12
   TXT_Size_ = 12
  Case Upper(comm_) = "PITCH_12"
   ret_val = ESC + "(s12h10v"            && Also sets height to 10
   TXT_Size_ = 10
  Case Upper(comm_) = "PITCH_17"
   ret_val  = ESC + "(s16.66h8.5v"       && Also sets height to 8.5
   TXT_Size_ = 8.5
  Otherwise
   ret_val = ""

Endcase
null=LP_Out(ret_val)
Return ""

* ---------------------------------------------------------

Procedure L_LPI
Parameters lpi_num

null=LP_Out(ESC + "&l" + ALLTRIM(STR(lpi_num)) + "D")
Return ""

* ---------------------------------------------------------

Procedure L_VLine
Parameters _line,_col,_len,_thick
Private line_,col_,len_,thick_,curs_,spec_,prin_

   line_   = V_AdjR(_line)
   col_    = V_AdjC(_col)
   len_    = LP_INDP(_len)
   thick_  = LP_INDP(_thick)

   curs_   = ESC + "&a" + line_ + "v" + col_ + "H"
   spec_   = ESC + "*c" + len_ + "v" + thick_ + "H"
   prin_   = ESC + "*c" + "0P"

Return curs_ + spec_ + prin_

* ---------------------------------------------------------

Procedure L_RVLine
Parameters _len,_thick

   _len    = LP_INDP(_len)
   _thick  = LP_INDP(_thick)

   spec_   = ESC + "*c" + _len + "v" + _thick + "H"
   prin_   = ESC + "*c" + "0P"

Return spec_ + prin_

* ---------------------------------------------------------

Procedure L_HLine
Parameters _line,_col,_len,_thick
Private line_,col_,len_,thick_,curs_,spec_,prin_

   line_   = V_AdjR(_line)
   col_    = V_AdjC(_col)
   len_    = LP_INDP(_len)
   thick_  = LP_INDP(_thick)

   curs_   = ESC + "&a" + line_ + "v" + col_ + "H"
   spec_   = ESC + "*c" + thick_ + "v" + len_ + "H"
   prin_   = ESC + "*c" + "0P"

Return  curs_ + spec_ + prin_

* ---------------------------------------------------------

Procedure L_RHLine
Parameters _len,_thick

   _len    = LP_INDP(_len)
   _thick  = LP_INDP(_thick)

   spec_   = ESC + "*c" + _thick + "v" + _len + "H"
   prin_   = ESC + "*c" + "0P"

Return  spec_ + prin_

* ---------------------------------------------------------

Procedure L_Grid
Parameters top_row,bottom_row,left_col,right_col,_grid

   ** Use as: =L_GRID(top_row_start, bottom_row_start, left_column, ;
   **                  right_column, type_of_grid)
   **
   ** Avilable grid types:  1 = Horizontal lines
   **                       2 = Vertical lines
   **                       3 = Diagonal lines 1
   **                       4 = Diagonal lines 2
   **                       5 = Square grid
   **                       6 = Diagonal grid

   _height = (bottom_row - top_row )
   _width  = (right_col  - left_col)

   _row_   = LTRIM(STR(((  top_row ) * 720 ) , 5 ))
   _col_   = LTRIM(STR((( left_col ) * 720 ) , 5 ))
   _high_  = LTRIM(STR(( _height * 720 ) , 5 ))
   _len_   = LTRIM(STR((  _width * 720 ) , 5 ))

   loc_    = ESC   + "&a"  + _row_  + "v" + _col_ + "H"
   info_   = ESC   + "*c"  + _high_ + "v" + _len_ + "H"
   grid_   = ESC   + "*c"  + STR(_grid,2) + "G"
   last_   = ESC   + "*c"  + "3P"

null=LP_Out(loc_  + info_ + grid_ + last_)
Return ""

* ---------------------------------------------------------

Procedure L_BoxShade
Parameters top_row,bottom_row,left_col,right_col,_shading
Private _row_,_col_,_high_,_len_,loc_,info_,shad_,last_

   _row_   = LTRIM(V_AdjR(top_row))
   _col_   = LTRIM(V_AdjC( left_col))
   _high_  = LP_INDP(bottom_row - top_row)
   _len_   = LP_INDP(right_col - left_col)

   loc_    = ESC   + "&a"  + _row_  + "v" + _col_ + "H"
   info_   = ESC   + "*c"  + _high_ + "v" + _len_ + "H"
   shad_   = ESC   + "*c"  + STR(_shading,3) + "G"
   last_   = ESC   + "*c"  + "2P"

Return loc_  + info_ + shad_ + last_

* ---------------------------------------------------------

Procedure L_RBoxShade
Parameters yval,xval,_shading
Private _row_,_col_,info_,shad_,last_

   _row_   = LTRIM(V_RAdjR(yval))
   _col_   = LTRIM(V_RAdjC(xval))

   info_   = ESC   + "*c"  + _row_ + "v" + _col_ + "H"
   shad_   = ESC   + "*c"  + STR(_shading,3) + "G"
   last_   = ESC   + "*c"  + "2P"

Return info_ + shad_ + last_

* ---------------------------------------------------------

Procedure L_SayIt
Parameters _down,_over,_txt,_pict
Private test_,_row,_col

   if type("_pict") = "C"
     text_ = LTRIM(TRANSFORM(_txt,_pict))
   else
     text_ = LTRIM(_txt)
   endif

   _row    = V_AdjR( _down)
   _col    = V_AdjC(_over)

Return ESC + "&a" + _row + "v" + _col + "H" + text_

* ---------------------------------------------------------

Procedure L_RSayIt
Parameters _txt,_pict
   if type("_pict") = "C"
     text_ = LTRIM(TRANSFORM(_txt,_pict))
   else
     text_ = LTRIM(_txt)
   endif

Return text_

* ---------------------------------------------------------

Procedure L_FSayIt
Parameters f_type, _down , _over , _ttext, _pict
Private tdown_,mxpos,_temp,mypos

Do Case
  Case Upper(f_type) = "STACKED"
    _ttext = LTRIM(TRANSFORM(_ttext,_pict))
    tdown_ = V_AdjR(_down)
    mxpos   = V_AdjC(_over)
    _temp = ""
    For i = 1 to len(trim(_ttext))
      mypos   = str(val(tdown_) + ((i-1) * ((TXT_Size_ * 10) * 1.05)))
      _temp = _temp + ESC + "&a" + mypos + "v" + mxpos + "H" + substr(_ttext,i,1)+crlf
    Endfor
Endcase

Return _temp

* ---------------------------------------------------------

Procedure L_MacroID
Parameters id_
* assigns a number a macro, between 0 and 32767
* also specifies a macro id for use in subsequent macro commands (last ID)

null=LP_Out(ESC + "&f" + LTRIM(STR(id_,10)) + CHR(89))
Return ""

* ---------------------------------------------------------

Procedure L_MacroCtl
Parameters func_
* 0 - Start macro definition
* 1 - Stop macro definition
* 2 - Execute macro (last ID)
* 3 - Call macro (last ID)
* 4 - Enable macro for automatic overlay (last ID)
* 5 - Disable automatic overlay
* 6 - Delete all macros
* 7 - Delete all temporary macros
* 8 - Delete macro (last ID)
* 9 - Make macro temporary (last ID)
* 10 - Make macro permanent (last ID)

null=LP_Out(ESC + "&f" + LTRIM(STR(func_,10)) + CHR(88))

Return ""

* ---------------------------------------------------------

Procedure L_LineLoop
Parameters LineMax,StartLine,StartCol,LineWidth,_Lpi

   ** Use as: Do LineLoop WITH max_lines, start_line, start_column,
   **                          line_width, lines_per_inch

   i = 1
   height_ = ROUND(1/_lpi,3)
   Do While (i*height_) <= LineMax
      ?? L_HLINE ( ( (i*height_) + StartLine) , StartCol , LineWidth, 1 )
      i = i + 1
   Enddo

Return ""

* ---------------------------------------------------------
* Section........: PostScript
* Version........: 0.2
* Author.........: Richard Elliott, Ferret Software
* Author ........: Tim Harvey, HALLoGRAM Publishing
* Copyright......: Copyright 1991, Ferret Software, All Rights reserved
* Purpose........: Postscript Procedure Library

* ---------------------------------------------------------

Procedure P_Orient
Parameters _orient

   Do Case
      Case "PORTRAIT" = UPPER( _orient )
         _temp = "0 0 translate 0 rotate"
      Case "LANDSCAPE" = UPPER( _orient )
         _temp = "612 0 translate 90 rotate"
        If LP_Layout_ = 1 .or. LP_Layout_ = 3
          PG_Size_ = 8.5
        Else
          PG_Size_ = 612
        Endif
      Otherwise
         _temp = ""
   Endcase
   null=LP_Out(_temp + crlf)
Return ""

* ---------------------------------------------------------

Procedure P_Lpi
Parameters lpi_num

   ** Defines /newline with

   line_size = STR(72/lpi_num,2,2)
   null=LP_Out("/newline")
   null=LP_Out("  {/ypos ypos &line_size sub def")
   null=LP_Out("  0 xpos ypos moveto} def" + crlf)

Return ""

* ---------------------------------------------------------
Procedure P_CurPos
Parameters row_,col_
Private rowc_,colc_

rowc_   = V_AdjR(row_)
colc_    = V_AdjC(col_)

Return colc_ + " " + rowc_ + " moveto" + crlf

* ---------------------------------------------------------
Procedure P_SRCurPos
Parameter SAVREST

* if SAVREST is .t. push current position, if .f. pop current position
if SAVREST
  temp_ = "gsave"
else
  temp_ = "grestore"
endif

Return temp_ + crlf

* ---------------------------------------------------------

Procedure P_RCurPos
Parameters row_,col_
Private rowc_,colc_

rowc_   = V_RAdjR(row_)
colc_    = V_RAdjC(col_)

Return colc_ + " " + rowc_ + " rmoveto" + crlf

* ---------------------------------------------------------

Procedure P_RCPos
Parameters row_,col_

Return " "

* ---------------------------------------------------------

Procedure P_FontPick
Parameters _font_, _size_

   TXT_Size_ = _size_
   points = ALLTRIM(STR(_size_,5,1))

   Do Case
      Case Upper(_font_) = "HEN"
         _temp  = "/Helvetica findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HEO"
         _temp  = "/Helvetica-Oblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HEB"
         _temp  = "/Helvetica-Bold findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HEX"
         _temp  = "/Helvetica-BoldOblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "TRN"
         _temp  = "/Times-Roman findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "TRI"
         _temp  = "/Times-Italic findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "TRB"
         _temp  = "/Times-Bold findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "TRX"
         _temp  = "/Times-BoldItalic findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "CRN"
         _temp  = "/Courier findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "CRO"
         _temp  = "/Courier-Oblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "CRB"
         _temp  = "/Courier-Bold findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "CRX"
         _temp  = "/Courier-BoldOblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "AGN"
         _temp  = "/AvantGarde-Book findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "AGO"
         _temp  = "/AvantGarde-BookOblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "AGD"
         _temp  = "/AvantGarde-Demi findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "AGX"
         _temp  = "/AvantGarde-DemiOblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "BKL"
         _temp  = "/Bookman-Light findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "BKI"
         _temp  = "/Bookman-LightItalic findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "BKD"
         _temp  = "/Bookman-Demi findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "BKX"
         _temp  = "/Bookman-DemiItalic findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HNN"
         _temp  = "/Helvetica-Narrow findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HNO"
         _temp  = "/Helvetica-Narrow-Oblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HNB"
         _temp  = "/Helvetica-Narrow-Bold findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "HNX"
         _temp  = "/Helvetica-Narrow-BoldOblique findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "NCN"
         _temp  = "/NewCenturySchlbk-Roman findfont " + points + " scalefont setfont"
      Case Upper(_font_) = "NCI"
         _temp  = "/NewCenturySchlbk-Italic findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "NCB"
         _temp  = "/NewCenturySchlbk-Bold findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "NCX"
         _temp  = "/NewCenturySchlbk-BoldItalic findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "PAN"
         _temp  = "/Palatino-Roman findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "PAI"
         _temp  = "/Palatino-Italic findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "PAB"
         _temp  = "/Palatino-Bold findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "PAX"
         _temp  = "/Palatino-BoldItalic findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "ZCM"
         _temp  = "/ZapfChancery-MediumItalic findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "ZAD"
         _temp  = "/ZapfDingbats findfont " + points + "  scalefont setfont"
      Case Upper(_font_) = "SYM"
         _temp  = "/Symbol findfont " + points + "  scalefont setfont"
      Otherwise
         _temp  = ""
   Endcase
   null=LP_Out(_temp + crlf)
Return ""

* ---------------------------------------------------------

Procedure P_SayIt
Parameters _down , _over , _ttext, _pict
Private _downc,_overc,_temp,mxpos,mypos

   if type("_pict") = "C"
     _ttext = LTRIM(TRANSFORM(_ttext,_pict))
   else
     _ttext = LTRIM(_ttext)
   endif
   _downc   = V_AdjR(_down)
   _overc   = V_AdjC(_over)
   mypos   = _downc
   mxpos   = _overc
   _temp   = mxpos + " " + mypos + " moveto" + crlf
   _temp   = _temp + "(" + _ttext + ") show"

Return _temp + crlf

* ---------------------------------------------------------

Procedure P_RSayIt
Parameters _ttext, _pict

if type("_pict") = "C"
  _ttext = LTRIM(TRANSFORM(_ttext,_pict))
endif
   _temp   = "(" + _ttext + ") show"

Return _temp + crlf

* ---------------------------------------------------------

Procedure P_FSayIt
Parameters f_type, _down , _over , _ttext, _pict
Private tdown_,mxpos,_temp,i,mypos

Do Case
  Case Upper(f_type) = "STACKED"
     _ttext = LTRIM(TRANSFORM(_ttext,_pict))
     tdown_ = V_AdjR(_down)
     mxpos   = V_AdjC(_over)
     _temp = ""
     For i = 1 to len(trim(_ttext))
       mypos   = str(val(tdown_) - ((i-1) * (TXT_Size_ * 1.05)))
       _temp   = _temp + mxpos + " " + mypos + " moveto" + crlf
       _temp   = _temp + "(" + substr(_ttext,i,1) + ") show"
     Endfor
Endcase

Return _temp + crlf

* ---------------------------------------------------------

Procedure P_SetGray
Parameters gray_

   ** 100 = Black, 0 = white, 99 - 1 = gray shades
   ** This also impacts the fonts and line/box drawing

   gray_ = 1-(gray_ * .01)
   _gray = ALLTRIM(STR( gray_, 5,2 ))
   _temp = _gray + " setgray"
   null=LP_Out(_temp + crlf)

Return ""

* ---------------------------------------------------------

Procedure P_LineDraw
Parameters _sline , _scol ,_eline , _ecol , _thick
Private sline_,scol_,eline_,ecol_,thick_,_temp

   sline_   = V_ADJR(_sline)
   scol_    = V_ADJC(_scol)
   eline_   = V_ADJR(_eline)
   ecol_    = V_ADJC(_ecol)
   thick_   = LP_PTIN(_thick)

   _temp = "newpath" + crlf
   _temp = _temp + "   " + scol_ + " " + sline_ + " moveto" + crlf
   _temp = _temp + "   " + ecol_ + " " + eline_ + " lineto" + crlf
   _temp = _temp + "   " + thick_  + " " + " setlinewidth" + crlf
   _temp = _temp + "stroke"

Return _temp + crlf

* ---------------------------------------------------------

Procedure P_BoxDraw
Parameters _sline , _scol ,_width , _height , _thick
Private sline_,scol_,width_,height_,thick_,_temp

   sline_   = V_ADJR(_sline)
   scol_    = V_ADJC(_scol)
   width_   = LP_PTIN(_width)
   height_  = LP_PTIN(_height)
   thick_   = LP_PTIN(_thick)

   _temp = "newpath" + crlf
   _temp = _temp + "   " + scol_ + " " + sline_ + " moveto" + crlf
   _temp = _temp + "   " + RIGHT( width_, 5) + "    0" + " rlineto" + crlf
   _temp = _temp + "   " + "   0 " + SPACE(5-LEN(ALLTRIM( height_ ))-1) + ;
                       "-" + ALLTRIM(height_) + " rlineto" + crlf
   _temp = _temp + "   " + SPACE(5-LEN(ALLTRIM( width_ ))-1) +"-"+ ;
                       ALLTRIM( width_ ) + "    0 rlineto " + crlf
   _temp = _temp + "   " + "closepath" + crlf
   _temp = _temp + "   " + thick_ + " setlinewidth" + crlf
   _temp = _temp + "stroke"

Return _temp + crlf

* ---------------------------------------------------------

Procedure P_RBoxDraw
Parameters _height , _width , _thick

   _width   = LP_PTIN(_width)
   _height  = LP_PTIN(_height)
   _thick   = LP_PTIN(_thick)

   _temp = "newpath" + crlf
   _temp = _temp + "   " + RIGHT( _width, 5) + "    0" + " rlineto" + crlf
   _temp = _temp + "   " + "   0 " + SPACE(5-LEN(ALLTRIM( _height ))-1) + ;
                       "-" + ALLTRIM(_height) + " rlineto" + crlf
   _temp = _temp + "   " + SPACE(5-LEN(ALLTRIM( _width ))-1) +"-"+ ;
                       ALLTRIM( _width ) + "    0 rlineto " + crlf
   _temp = _temp + "   " + "closepath" + crlf
   _temp = _temp + "   " + _thick + " setlinewidth" + crlf
   _temp = _temp + "stroke"

Return _temp + crlf

* ---------------------------------------------------------

Procedure P_BoxShade
Parameters _sline , _scol ,_width , _height , _gray
Private _slinec,_scolc,_widthc,_heightc,_grayc,_temp

   ** 100 = Black, 0 = white, 99 - 1 = gray shades
   ** This also impacts the fonts and line/box drawing

   _gray = 1-(_gray * .01)
   _grayc = ALLTRIM(STR( _gray, 5,2 ))

   _slinec   = V_ADJR(_sline)
   _scolc    = V_ADJC(_scol)
   _widthc   = LP_PTIN(_width)
   _heightc  = LP_PTIN(_height)

   _temp = "newpath" + crlf
   _temp = _temp + "   gsave" + crlf
   _temp = _temp + "   " + _scolc + " " + _slinec + " moveto" + crlf
   _temp = _temp + "   " + RIGHT(_widthc,5) + "    0 rlineto" + crlf
   _temp = _temp + "      0 " + SPACE(5-LEN(ALLTRIM(_heightc))-1) + "-"+ ;
                       ALLTRIM(_heightc)  + " rlineto" + crlf
   _temp = _temp + "   " + SPACE(5-LEN(ALLTRIM(_widthc))-1) + "-" + ;
                       ALLTRIM(_widthc) + "    0 rlineto" + crlf
   _temp = _temp + "   closepath" + crlf
   _temp = _temp + "   " + _grayc + " setgray" + crlf
   _temp = _temp + "   fill" + crlf
   _temp = _temp + "   grestore"

Return _temp + crlf

* ---------------------------------------------------------
Procedure P_RBoxShade
Parameters _height , _width , _gray
Private _grayc,_widthc,_heightc,_temp

   ** 100 = Black, 0 = white, 99 - 1 = gray shades
   ** This also impacts the fonts and line/box drawing

   _gray = 1-(_gray * .01)
   _grayc = ALLTRIM(STR( _gray, 5,2 ))

   _widthc   = V_RAdjC(_width)
   _heightc  = V_RAdjR(_height)

   _temp = "   gsave" + crlf
   _temp = _temp + "   " + RIGHT(_widthc,5) + "    0 rlineto" + crlf
   _temp = _temp + "      0 " + SPACE(5-LEN(ALLTRIM(_heightc))-1) + ;
                       iif(at("-",_heightc) > 0,"","-") + ;
                       ALLTRIM(_heightc)  + " rlineto" + crlf
   _temp = _temp + "   " + SPACE(5-LEN(ALLTRIM(_widthc))-1) + ;
                       iif(at("-",_widthc) > 0,"","-") + ;
                       ALLTRIM(_widthc) + "    0 rlineto" + crlf
   _temp = _temp + "   closepath" + crlf
   _temp = _temp + "   " + _grayc + " setgray" + crlf
   _temp = _temp + "   fill" + crlf
   _temp = _temp + "   grestore"

Return _temp + crlf

* ---------------------------------------------------------

*********************************************************************

*********************************************************************
* Dot Matrix Printer Driver Code Section
*********************************************************************

Procedure BC_EPSPBC

* Default of 240 dpi, using 8 pin image quadruple density, set
* BC_Gmode to "Z" and BC_Pinw to .004
* To use 120 dpi, 8 pin double density set BC_Gmode to "L" and
* BC_Pinw to .008
BC_Gmode = "Z"
BC_Gcols = 0
BC_GcolsG = 0
BC_PCode = " "
BC_PCodeG = " "

* set y positioning
if BC_CStyle[29] = 0
  bc_rdt = set("DEVICE")
  bc_rpt = set("PRINTER")
  Set Device to Print
  if bc_rpt = "OFF"
    Set Print on
  endif
  @ BC_CStyle[3],1 say " "
  if bc_rpt = "OFF"
    Set Print off
  endif
  set device to &bc_rdt
endif


do case
case BC_CStyle[1] = "CODE39"

  BC_Gcols = ccode39(BC_Gmode,BC_MPrt,@BC_PCode)

  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor
  for h = 2 to iif(BC_CStyle[11] = 1,BC_CStyle[8] - 1,BC_CStyle[8])
    null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
           mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
  endfor
  if BC_CStyle[11] = 1
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[22] + BC_CStyle[23]) + chr(13))
  endif
case BC_CStyle[1] = "INTERLEAVED25"

  BC_Gcols = interl25(BC_Gmode,BC_MPrt,@BC_PCode)

  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor
  for h = 2 to iif(BC_CStyle[11] = 1,BC_CStyle[8] - 1,BC_CStyle[8])
    null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
           mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
  endfor
  if BC_CStyle[11] = 1
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[22] + BC_CStyle[23]) + chr(13))
  endif

case BC_CStyle[1] = "CODE128"

  BC_Gcols = ccode128(BC_Gmode,BC_MPrt,@BC_PCode)

  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor
  for h = 2 to iif(BC_CStyle[11] = 1,BC_CStyle[8] - 1,BC_CStyle[8])
    null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
           mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
  endfor
  if BC_CStyle[11] = 1
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[22] + BC_CStyle[23]) + chr(13))
  endif

case BC_CStyle[1] = "UPC_A"

  BC_GcolsG = gbarupca(BC_Gmode,BC_MPrt,@BC_PCodeG)
  BC_Gcols = ccodeupca(BC_Gmode,BC_GPrt,@BC_PCode)

  * Print guard bar tops
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_GcolsG,BC_PCodeG)
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor

  BC_Gcols = ccodeupca(BC_Gmode,BC_MPrt,@BC_PCode)

  for h = 2 to iif(BC_CStyle[11] = 1,BC_CStyle[8] - 1,BC_CStyle[8]) - 1
    null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
           mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
  endfor

  null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
         mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor
  null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))

  BC_Gcols = ccodeupca(BC_Gmode,BC_GTPrt,@BC_PCode)

  * Print guard bar tops
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
    null=LP_Out(chr(13))
    null=BC_PLine(BC_GcolsG,BC_PCodeG)
  endfor
  if BC_CStyle[11] = 1
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[22] + BC_CStyle[23]) + chr(13))
  endif

case BC_CStyle[1] = "UPC_E"

  BC_GcolsG = gbarupce(BC_Gmode,BC_MPrt,@BC_PCodeG)
  BC_Gcols = ccodeupce(BC_Gmode,BC_GPrt,@BC_PCode)

  * Print guard bar tops
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_GcolsG,BC_PCodeG)
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor

  BC_Gcols = ccodeupce(BC_Gmode,BC_MPrt,@BC_PCode)

  for h = 2 to iif(BC_CStyle[11] = 1,BC_CStyle[8] - 1,BC_CStyle[8]) - 1
    null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
           mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))
    for tk = 1 to BC_CStyle[24]
      null=LP_Out(chr(13))
      null=BC_PLine(BC_Gcols,BC_PCode)
    endfor
  endfor

  null=LP_Out(chr(27) + "J" + chr(iif(mod(BC_CStyle[18],BC_CStyle[17])>0 .and. ;
         mod(h,2)>0,BC_CStyle[22]+1,BC_CStyle[22])) + chr(13))
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor
  null=LP_Out(chr(27) + "J" + chr(BC_CStyle[23]) + chr(13))

  BC_Gcols = ccodeupce(BC_Gmode,BC_GTPrt,@BC_PCode)

  * Print guard bar tops
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
    null=LP_Out(chr(13))
    null=BC_PLine(BC_GcolsG,BC_PCodeG)
  endfor
  if BC_CStyle[11] = 1
    null=LP_Out(chr(27) + "J" + chr(BC_CStyle[22] + BC_CStyle[23]) + chr(13))
  endif

case BC_CStyle[1] = "POSTNET"

  BC_Gcols = ccodepnet(BC_Gmode,@BC_PCode)
  for tk = 1 to BC_CStyle[24]
    null=LP_Out(chr(13))
    null=BC_PLine(BC_Gcols,BC_PCode)
  endfor

endcase

Return ""

*********************************************************************

Procedure BC_EPSPTXT

if BC_CStyle[16] = "EPSON"
  * Select 10 CPI
  null=LP_Out(chr(27) + "P")
  BC_Chw = 0.1
else
  * Select 10 CPI
  null=LP_Out(chr(27) + chr(18))
  BC_Chw = 0.1
endif

do case
  case BC_CStyle[1] = "CODE39"
    do case
      case BC_CStyle[13] = "CENTER" .or. BC_CStyle[13] = "SPLIT"
        BC_Tx = int(((((len(BC_Messg)+2) / BC_CStyle[7])/2 - ;
                     (BC_Chw * len(BC_Messg))/2) / BC_Chw))
      case BC_CStyle[13] = "RIGHT"
        BC_Tx = int(((((len(BC_Messg)+2) / BC_CStyle[7]) - ;
                     (BC_Chw * len(BC_Messg))) / BC_Chw))
      otherwise
        BC_Tx = 0
    endcase

    null=LP_Out(space(BC_CStyle[2] + BC_Tx)+BC_Messg+chr(13))

  case BC_CStyle[1] = "INTERLEAVED25"
    do case
      case BC_CStyle[13] = "CENTER" .or. BC_CStyle[13] = "SPLIT"
        BC_Tx = int(((((len(BC_Messg) * 8 * BC_CStyle[20]) + (BC_Cstyle[20] * 9))/2);
                 - (BC_Chw * len(BC_Messg))/2)/BC_Chw)
      case BC_CStyle[13] = "RIGHT"
        BC_Tx = int((((len(BC_Messg) * 8 * BC_CStyle[20]) + (BC_Cstyle[20] * 9));
                 - (BC_Chw * len(BC_Messg)))/BC_Chw)
      otherwise
        BC_Tx = 0
    endcase

    null=LP_Out(space(BC_CStyle[2] + BC_Tx)+BC_Messg+chr(13))

  case BC_CStyle[1] = "CODE128"
    * calculate the length of the barcode
    cont = .t.
    i = 1
    m128len = 0
    m128tlen = 0
    bandc = .f.
    do while cont
      t = substr(BC_Messg,i,2)
      if t = chr(128) + chr(128)
        if substr(BC_Messg,i+2,1) = chr(3) .or. substr(BC_Messg,i+2,1) = chr(9)
          bandc = .t.
        else
          if substr(BC_Messg,i+2,1) = chr(4) .or. substr(BC_Messg,i+2,1) = chr(5)
            bandc = .f.
          endif
        endif
        m128len = m128len + 1
        i = i + 3
      else
        m128len = m128len + 1
        m128tlen = m128tlen + iif(bandc,2,1)
        i = i + iif(bandc,2,1)
      endif
      if i+2 > len(BC_Messg)
        m128len = m128len + iif(bandc,1,(len(BC_Messg) - i + 1))
        m128tlen = m128tlen + iif(bandc,2,(len(BC_Messg) - i + 1))
        cont = .f.
      endif
    enddo

    do case
      case BC_CStyle[13] = "CENTER" .or. BC_CStyle[13] = "SPLIT"
        BC_Tx = int(((((m128len * 11 * BC_CStyle[20]) + (BC_Cstyle[20] * 24))/2);
                 - (BC_Chw * m128tlen)/2)/BC_Chw)
      case BC_CStyle[13] = "RIGHT"
        BC_Tx = int(((m128len * 11 * BC_CStyle[20]) + (BC_Cstyle[20] * 24));
                 - (BC_Cl * m128tlen)/BC_Chw)
      otherwise
        BC_Tx = 0
    endcase

    null=LP_Out(space(BC_CStyle[2] + BC_Tx)+BC_Messg+chr(13))

  case BC_CStyle[1] = "UPC_A"
    BC_CStyle[20] = BC_CStyle[7]/95
    do case
      case BC_CStyle[13] = "SPLIT"
        BC_Tx1 = round((((24 * BC_CStyle[20]) - (BC_Chw * 3)) / BC_Chw), 0)
        BC_Tx2 = round((((71 * BC_CStyle[20]) - (BC_Chw * 3)) / BC_Chw), 0)
        null=LP_Out(space(BC_CStyle[2] + BC_Tx1)+substr(BC_Messg,1,6)+;
        space(iif(BC_Tx2 - BC_Tx1 - 6<1,1,BC_Tx2 - BC_Tx1 - 6))+;
              substr(BC_Messg,7,6)+chr(13))
      otherwise
        BC_Tx1 = round((((48 * BC_CStyle[20]) - (BC_Chw * 6)) / BC_Chw), 0)
        null=LP_Out(space(BC_CStyle[2] + BC_Tx1)+BC_Messg+chr(13))
    endcase
  case BC_CStyle[1] = "UPC_E"
    BC_CStyle[20] = BC_CStyle[7]/51
    BC_Tx = round((((24 * BC_CStyle[20]) - (BC_Chw * 3)) / BC_Chw), 0)
    BC_Tmp = BC_CStyle[2] + BC_Tx
    null=LP_Out(space(BC_CStyle[2] + BC_Tx)+BC_Messg+chr(13))
endcase
return ""

*********************************************************************

Procedure MkCodeStr
parameters codestr,dgt,BC_Cnt,BC_GCols,pval

BC_Pinw = iif(BC_CStyle[1]="CODE39",.0035,.004)
BC_Narrow = int(BC_CStyle[20]/BC_Pinw * ;
             iif(dgt,BC_CStyle[33],1+(1-BC_CStyle[33])))

for i = 1 to (BC_Narrow * BC_Cnt)
  codestr = codestr + iif(dgt,chr(pval),chr(0))
  BC_GCols = BC_GCols + 1
endfor
return ""

*********************************************************************

Procedure MkPostStr
parameters codestr,dgt,BC_Cnt,BC_GCols

BC_Pinw = iif(BC_CStyle[1]="CODE39",.0035,.004)
BC_Narrow = int(BC_CStyle[20]/BC_Pinw * ;
             iif(dgt>0,BC_CStyle[33],1+(1-BC_CStyle[33])))

for i = 1 to (BC_Narrow * BC_Cnt)
  codestr = codestr + iif(dgt = 0,chr(0),iif(dgt = 1, chr(255),chr(15)))
  BC_GCols = BC_GCols + 1
endfor
return ""

*********************************************************************
Procedure BC_PLine
parameters BC_Gcols,codestr

* define n1 and n2
n1 = BC_Gcols % 256
n2 = int(BC_Gcols/256)
null=LP_Out(space(BC_CStyle[2])+chr(27) + BC_GMode + chr(n1) + chr(n2) + codestr)

return
*********************************************************************

